unit DM_MME;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, ZConnection, ZDataset, ZSqlMonitor, db;

type

  { TCategoryItem }
  TCategoryItem = class
  private
    FCategory: string;
    FInDB: Boolean;
    FSubCategory: string;
    function GetCategoryDisplay: string;
    procedure SetCategory(AValue: string);
    procedure SetInDB(AValue: Boolean);
    procedure SetSubCategory(AValue: string);
  published
    property Category : string read FCategory write SetCategory;
    property SubCategory : string read FSubCategory write SetSubCategory;
    property InDB : Boolean read FInDB write SetInDB;
    property CategoryDisplay: string read GetCategoryDisplay;
  end;

  { TDmMME }
  TDmMME = class(TDataModule)
    DS_MMECat: TDataSource;
    ZConMME: TZConnection;
    ZQMME_Categories: TZQuery;
    ZQMME_CategoriesCATEGID: TLargeintField;
    ZQMME_CategoriesCATEGNAME: TMemoField;
    ZQMME_CategoriesSUBCATEGID: TLargeintField;
    ZQMME_CategoriesSUBCATEGNAME: TMemoField;
    ZSQLMonitor1: TZSQLMonitor;
    procedure DataModuleCreate(Sender: TObject);
    procedure DS_MMECatDataChange(Sender: TObject; Field: TField);
  private
    FDBName: string;
    procedure SetDBName(AValue: string);
    { private declarations }
  public
    { public declarations }
    function GetAllCategoryItems:TStringList;
    property DBName: string read FDBName write SetDBName;
  end;


//Singleton
function gDmMEE : TDmMME;

implementation

{$R *.lfm}

var
  DmMME: TDmMME;

function gDmMEE: TDmMME;
begin
  if not assigned(DmMME) then
    DmMME := TDmMME.Create(nil);
  Result := DmMME;
end;

{ TCategoryItem }

procedure TCategoryItem.SetCategory(AValue: string);
begin
  if FCategory=AValue then Exit;
  FCategory:=AValue;
end;

function TCategoryItem.GetCategoryDisplay: string;
begin
  if SubCategory = '' then
    Result:= Category
  else
    Result := Category + '/' + SubCategory;
end;

procedure TCategoryItem.SetInDB(AValue: Boolean);
begin
  if FInDB=AValue then Exit;
  FInDB:=AValue;
end;

procedure TCategoryItem.SetSubCategory(AValue: string);
begin
  if FSubCategory=AValue then Exit;
  FSubCategory:=AValue;
end;

{ TDmMME }

procedure TDmMME.DataModuleCreate(Sender: TObject);
begin

end;

procedure TDmMME.DS_MMECatDataChange(Sender: TObject; Field: TField);
begin

end;

procedure TDmMME.SetDBName(AValue: string);
begin
  if FDBName=AValue then Exit;
  if ZConMME.Connected then ZConMME.Disconnect;
  ZConMME.Database:=AValue;
  FDBName:=AValue;
end;

function TDmMME.GetAllCategoryItems: TStringList;
var
  AItem : TCategoryItem;
  CName : string;
  SCName : string;
begin
  Result := TStringList.Create;
  if not ZConMME.Connected then ZConMME.Connect;
  ZQMME_Categories.Open;
  ZQMME_Categories.First;
  while not ZQMME_Categories.EOF do
  begin
    CName:=ZQMME_Categories.FieldByName('CATEGNAME').AsString;
    SCName := ZQMME_Categories.FieldByName('SUBCATEGNAME').AsString;
    AItem := TCategoryItem.Create;
    AItem.Category:=CName;
    AItem.SubCategory:=SCName;
    Result.AddObject(AItem.CategoryDisplay,AItem);
    ZQMME_Categories.Next;
  end;
  ZQMME_Categories.Close;
  ZConMME.Disconnect;
end;

initialization

finalization
  DmMME.Free;

end.

