unit fr_convert;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StdCtrls
  ;

function ConnectToDatabase(const lDB: string; InfoMemo :TMemo):boolean;
function CreateNewDatabase(const lDB: string; InfoMemo :TMemo):boolean;
function CreateNewRaika:boolean;
function DisconnectFromDatabase(InfoMemo :TMemo):Boolean;
function DoConvert(memoInfo, memoQuelle, memoZiel:TMemo; FileName:string =''; StartLine:Integer = 0): Boolean;
function DoEditFilter:Boolean;
function DoEditSearch:Boolean;
function DoEditHash:Boolean;

implementation

uses
  Dialogs
  , Controls
  , Forms
  , UTextFileStream
  , RegExpr
  , MD5
  , tiObject
  , tiUtils
  , tiOPFManager
  , tiConstants
  , tiMediators
  , tiLog
  , tiLogToFile
  , BOM_Filter
  , SUPPORT_Filter
  , FACTORY_Filter
  , BOM_Hash
  , FACTORY_Hash
  , SUPPORT_Hash
  , BOM_Search
  , SUPPORT_Search
  , uFilter
  , uListManager
  , ufrmeditfilter
  , ufrmeditsearch
  , ufrmeditHash
  ;

resourcestring
  rStrCantOpen = 'Cannot open';
  rStrTabelle = 'Table';
  rStrDoesntExist = 'doesnt exist';
  rStrExist = 'exist';
  rStrDatabase = 'Database';
  rStrCreateTableFail = 'Create table fail, table';

function ConnectToDatabase(const lDB: string; InfoMemo :TMemo):boolean;
begin
  Result := False;
  if not gTIOPFManager.DefaultPerLayer.DatabaseExists(lDB,  'null', 'null') then
  begin
    InfoMemo.Append(rStrCantOpen + ' ' + rStrDatabase);
    exit; //==>
  end;
  gTIOPFManager.TestThenConnectDatabase(lDB,  'null', 'null', cTIPersistSqldbSQLite3);
  if not gTIOPFManager.TableExists('Filters') then
  begin
    InfoMemo.Append(rStrTabelle + ' Filters ' + rStrDoesntExist);
    exit; //==>
  end;
  if not gTIOPFManager.TableExists('Hashes') then
  begin
    InfoMemo.Append(rStrTabelle + ' Hashes ' + rStrDoesntExist);
    exit; //==>
  end;
  if not gTIOPFManager.TableExists('Search') then
  begin
    InfoMemo.Append(rStrTabelle + ' Search ' + rStrDoesntExist);
    exit; //==>
  end;
  Result := true;
end;

function CreateNewDatabase(const lDB: string; InfoMemo :TMemo): boolean;
begin
  Result := False;
  if gTIOPFManager.DefaultPerLayer.DatabaseExists(lDB,  'null', 'null') then
  begin
    InfoMemo.Append(rStrDatabase + ' ' + lDB + ' ' + rStrExist);
    exit; //==>
  end;
  gTIOPFManager.TestThenConnectDatabase(lDB,  'null', 'null', cTIPersistSqldbSQLite3);
  if not gTIOPFManager.TableExists('Filters') then
  begin
    SUPPORT_Filter.CreateTables;
    if not gTIOPFManager.TableExists('Filters') then
    begin
      InfoMemo.Append(rStrCreateTableFail + ' Filters ' + rStrDoesntExist);
      exit; //==>
    end;
  end;
  if not gTIOPFManager.TableExists('Hashes') then
  begin
    SUPPORT_Hash.CreateTables;
    if not gTIOPFManager.TableExists('Hashes') then
    begin
      InfoMemo.Append(rStrCreateTableFail + ' Hashes ' + rStrDoesntExist);
      exit; //==>
    end;
  end;
  if not gTIOPFManager.TableExists('Search') then
  begin
    SUPPORT_Search.CreateTables;
    if not gTIOPFManager.TableExists('Search') then
    begin
      InfoMemo.Append(rStrCreateTableFail + ' Search ' + rStrDoesntExist);
      exit; //==>
    end;
  end;
  Result := true;
end;

function CreateNewRaika: boolean;
var
  myList : TFilterList;
  myFilter : TFilter;
begin
  myList := gListManager.FilterList;
  myList.Clear;
  myList.Save;
  // now we have an empty list in the DB
   myFilter := gFilterFactory.CreateInstance('FilterCopy');
//  myFilter := TFilterCopy.CreateNew();
  myFilter.InputFields:='0,1,2,4,5,6';
  myFilter.OutputFields:='0,1,2,4,5,6';
  myFilter.Params:='';
  myFilter.Remark:='Only Copy';
  myFilter.Dirty:=true;
  myList.Add(myFilter);

  myFilter := gFilterFactory.CreateInstance('FilterSearch');
//  myFilter := TFilterSearch.CreateNew();
  myFilter.InputFields:='1';
  myFilter.OutputFields:='8,9,10';
  myFilter.Params:='Search';
  myFilter.Remark:='Search for known entries';
  myFilter.Dirty:=true;
  myList.Add(myFilter);

  myFilter := gFilterFactory.CreateInstance('FilterConvertNumber');
//  myFilter := TFilterConvertNumber.CreateNew();
  myFilter.InputFields:='3';
  myFilter.OutputFields:='3';
  myFilter.Params:='",",".",".",","';
  myFilter.Remark:='convert the number';
  myFilter.Dirty:=true;
  myList.Add(myFilter);

  myFilter := gFilterFactory.CreateInstance('FilterHash');
//  myFilter := TFilterHash.CreateNew();
  myFilter.InputFields:='';
  myFilter.OutputFields:='7';
  myFilter.Params:='';
  myFilter.Remark:='Place for the hash';
  myFilter.Dirty:=true;
  myList.Add(myFilter);
  myList.Save;
end;

function DisconnectFromDatabase(InfoMemo: TMemo): Boolean;
begin
  if gTIOPFManager.DefaultPerLayer.DBConnectionPools.Count > 0 then
  begin
    gTIOPFManager.DisconnectDatabase;
    InfoMemo.Append('Disconnected from DB');
  end;
  Result:=true;
end;


function DoConvert(memoInfo, memoQuelle, memoZiel:TMemo; FileName:string =''; StartLine:Integer = 0): Boolean;
var
  TFS : TTextFileStream;
  TFSW : TTextFileStream;
  TS: TStringList;
  TSW: TStringList;
  OneLine : string;
  Hash : string;
  aHash : THash;
  LineCount : Integer;
  strTemp : String;
  aTiObjects : TtiObject;
begin
  Result := false;
  // Check prerequisites
  if FileName = '' then begin memoInfo.Append('FileName not set'); exit; end;
  if not FileExists(FileName) then begin memoInfo.Append('File not found:' + FileName ); exit; end;
  // All Ok
  TS:= TStringList.Create();
  TFS:= TTextFileStream.Create(FileName,fmOpenRead);
  TFSW:= TTextFileStream.Create(ChangeFileExt(FileName,'.exp.csv'),fmCreate);
  TSW := TStringList.Create();
  try
    LineCount := 0;
    while (not TFS.Eof) do
    begin
      inc(LineCount);
      if ((LineCount mod 10) = 0) then memoInfo.Append('Process line: '+IntToStr(LineCount));
      OneLine:= TFS.ReadLn;
      // Skip over line if startline not reached
      if (LineCount <= StartLine) then
      begin
        memoInfo.Append('Skip line: '+IntToStr(LineCount));
        Continue;
      end;
      // If line is empty, we skip this line
      if OneLine = '' then
      begin
        memoInfo.Append('Empty line: '+IntToStr(LineCount) + ' skipping');
        Continue;
      end;
      // Make Hash
      Hash := LowerCase(MDPrint(MDString(OneLine, MD_VERSION_5)));
//***********
      if gListManager.HashList.FindHash(Hash) <> 0 then
      begin
        memoInfo.Append('Duplicated line: '+IntToStr(LineCount) + ' skipping');
        Continue; //=>
      end;


      TS.Clear;
      TS.Delimiter:=';';
      TS.DelimitedText := Oneline;
      TSW.Clear;
      TSW.Delimiter:=';';
      gListManager.FilterList.Read;

      memoQuelle.Append(TS.DelimitedText);
      for aTiObjects in gListManager.FilterList do
      begin
        if TFilter(aTiObjects).CheckParams then
        begin
          if aTiObjects is TFilterHash then
          begin
            TFilterHash(aTiObjects).Params:=Hash;
          end;
          if not TFilter(aTiObjects).Execute(TS,TSW) then
            memoInfo.Append('Execute ' + TFilter(aTiObjects).ClassName + ' failed');;
        end
        else
          memoInfo.Append('Checkparams of ' + aTiObjects.ClassName + ' failed');
      end;
      memoZiel.Append(TSW.DelimitedText);
      aHash := THash.CreateNew;
      aHash.FileHash:=Hash;
      aHash.Filename:=FileName;
      aHash.StoreDate:=now();
      gListManager.HashList.Add(aHash);
      gListManager.HashList.Save;
      // Store Result
      strTemp := TSW.DelimitedText;
      if strTemp='' then strTemp :='nothing';
      TFSW.Write(strTemp[1], Length(strTemp));
      TFSW.Write(LineEnding, Length(LineEnding));
    end;
  finally
    TS.Free;
    TSW.Free;
    TFS.Free;
    TFSW.Free;
  end;
end;

function DoEditFilter: Boolean;
var
  ModRes : Boolean;
begin
  gListManager.FilterList.Read;
  //Result := ShowEditFilter(gListManager.FilterList);
  ModRes := TfrmEditFilter.Execute(gListManager.FilterList,false);
  Result := ModRes;
  gListManager.FilterList.Save;
end;

function DoEditSearch: Boolean;
var
  ModRes : Boolean;
begin
  gListManager.SearchList.Read;
//  Result :=  ShowEditSearch(gListManager.SearchList,'Dummy')=mrOK;
  ModRes := TfrmEditSearch.Execute(gListManager.SearchList,'Dummy',false);
  Result := ModRes;
  gListManager.SearchList.Save;
end;

function DoEditHash: Boolean;
var
  ModRes : Boolean;
begin
  gListManager.HashList.Criteria.ClearAll;
  gListManager.HashList.Clear;
  gListManager.HashList.Read;
  ModRes := TfrmEditHash.Execute(gListManager.HashList,false);
  Result := ModRes;
  gListManager.HashList.Save;
end;


initialization
  RegisterFallBackMediators;


end.

