unit frdb_main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs,
  LCLTranslator, StdCtrls, XMLPropStorage, ActnList, Menus, ComCtrls,
  LocalizedForms;

type

  { TForm1 }

  TForm1 = class(TLocalizedForm)
    actConvert: TAction;
    actCreateNewDatabase: TAction;
    actCreateNewDatabaseRaika: TAction;
    actEditFilter: TAction;
    actEditHash: TAction;
    actEditSearch: TAction;
    actGetFilename: TAction;
    actLangDE: TAction;
    actLangEN: TAction;
    actlFRMain: TActionList;
    actQuit: TAction;
    actSearchConfig: TAction;
    imglFRMain: TImageList;
    MainMenu1: TMainMenu;
    MemoInfo: TMemo;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    mnuConfigEdit: TMenuItem;
    mnuConfigLoad: TMenuItem;
    mnuEditFilter: TMenuItem;
    mnuEditHash: TMenuItem;
    mnuEditSearch: TMenuItem;
    mnuFileQuit: TMenuItem;
    mnuLangDeutsch: TMenuItem;
    mnuLangEnglisch: TMenuItem;
    mnuSubConfiguration: TMenuItem;
    mnuSubFile: TMenuItem;
    mnuSubSprache: TMenuItem;
    ODConfigName: TOpenDialog;
    StatusBar1: TStatusBar;
    XMLConfigMain: TXMLPropStorage;
    procedure actCreateNewDatabaseExecute(Sender: TObject);
    procedure actCreateNewDatabaseRaikaExecute(Sender: TObject);
    procedure actEditFilterExecute(Sender: TObject);
    procedure actEditHashExecute(Sender: TObject);
    procedure actEditSearchExecute(Sender: TObject);
    procedure actLangDEExecute(Sender: TObject);
    procedure actLangENExecute(Sender: TObject);
    procedure actQuitExecute(Sender: TObject);
    procedure actSearchConfigExecute(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure XMLConfigMainRestoreProperties(Sender: TObject);
    procedure XMLConfigMainSaveProperties(Sender: TObject);
  private
    FActualConverter: string;
    FLastLang: string;
    BaseFileName : String;
    AppFileCfgName : String;
    procedure SetActualConverter(AValue: string);
    procedure SetStatusPanel;
    procedure DoConfigSet;
  protected
    procedure PrepareTranslation;
    procedure SelectLanguage(ALang: String);
    procedure UpdateTranslation(ALang: String); override;
  public

  published
    property LastLang: string read FLastLang write FLastLang;
    property ActualConverter: string read FActualConverter write SetActualConverter;
  end;

var
  Form1: TForm1;

implementation

uses
  tiQuerySqldbSQLite3 // SQLite 3 with SqlDB is used
  , tiUtils
  , tiOPFManager
  , tiConstants
  , tiLog
  , tiLogToFile
  , LazFileUtils
  , fr_convert
  ;


{$R *.lfm}

resourcestring
   rStrMainCaption = 'AF Converter for CSV Files';

   rStractQuitCaption = 'Quit';
   rStractLangDECaption = 'Deutsch';
   rStractLangENCaption = 'English';
   rStractGetFilenameCaption = 'Search';
   rStractSearchConfigCaption = 'Load Configuration';
   rStractEditSearchCaption ='Edit Search';
   rStractEditHashCaption ='Edit Hash';
   rStractEditFilterCaption ='Edit Filter';
   rStractCreateNewDatabase = 'Create new Database';

   rStrmnuSubFileCaption = 'File ';
   rStrmnuSubSpracheCaption = 'Language';
   rStrmnuSubConfiguration = 'Configuration';

   rStrLblFileNameCaption = 'Sourcefile';

   rStrODFileNameTitle = 'Open existing file';
   rStrODFileNameFilter1 = 'Comma seperated values files|*.csv';
   rStrODFileNameFilter2 = 'Text files|*.txt';
   rStrODFileNameFilter3 = 'All files|*.*';

   rStrODConfigNameTitle = 'Open existing configuration';
   rStrODConfigNameFilter1 = 'Configuration files|*.sq3';

   rStractGetFilenameNoFile = 'File not found';
   rStractSearchConfigNoFile = 'Configuration not found';
   rStractSearchConfigFileExist = 'Error: Configuration exists';

   rStrErrorEditFilter = 'Error in Editfilter';
   rStrErrorDBEditFilter = 'Error can not open database';
   rStrErrorEditHash = 'Error in Edithash';
   rStrErrorDBEditHash = 'Error can not open database';
   rStrErrorEditSearch = 'Error in Editsearch';
   rStrErrorDBEditSearch = 'Error can not open database';

   rStrStatusBar1PanelsItems1Text = 'Config :';

   rStrNoFileLoaded = 'No config loaded';


procedure StartLog;
var
  aFileName : String;
  i : Integer;
  LS : String;
begin
  // Log
  aFileName := tiSwapExt(ParamStr(0),'log');
  gLog.RegisterLog(TtiLogToFile.CreateWithFileName(ExtractFilePath(aFileName),ExtractFileName(aFileName),True));
  gLog.SevToLog :=  [
    lsNormal
    ,lsUserInfo
    ,lsObjCreation
    ,lsVisitor
    ,lsConnectionPool
    ,lsAcceptVisitor
    ,lsQueryTiming
    ,lsDebug
    ,lsWarning
    ,lsError
    ,lsSQL
    ];
  Log('Starting', lsNormal);
  LS := '';
  for i := 0 to GTIOPFManager.PersistenceLayers.Count - 1 do
  begin
    if LS <> '' then
      LS := LS + cLineEnding;
    if Trim(GTIOPFManager.PersistenceLayers.Items[i].DBConnectionPools.DetailsAsString) = '' then
      LS := LS + 'Persistence layer: "'+ GTIOPFManager.PersistenceLayers.Items[i].PersistenceLayerName +
            '" loaded, but not connected to a database.'
    else
      LS := LS + GTIOPFManager.PersistenceLayers.Items[i].DBConnectionPools.DetailsAsString
  end;
  if LS <> '' then
    Log(LS, lsDebug)
  else
    Log('No persistence layers loaded', lsDebug);
end;



{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  inherited;
  GTIOPFManager.DefaultPersistenceLayerName := cTIPersistSqldbSQLite3;
  StartLog;
  LastLang:= GetDefaultLang;
  if LastLang = '' then
    LastLang:= 'de';
  SelectLanguage(LastLang);  // Spracheinstellungen laden
  SetStatusPanel;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //XMLConfigMainSaveProperties(Sender);
end;

procedure TForm1.actLangDEExecute(Sender: TObject);
begin
  LastLang := 'de';
  SelectLanguage(LastLang);
end;

procedure TForm1.actCreateNewDatabaseExecute(Sender: TObject);
begin
  DisconnectFromDatabase(MemoInfo);
  // Erzeugen einer neuen Konfiguration
  if ActualConverter <> '' then
    ODConfigName.InitialDir:=ExtractFilePath(ActualConverter)
  else
    ODConfigName.InitialDir:=ExtractFilePath(ParamStr(0));
  ODConfigName.DefaultExt:='.sq3';
  ODConfigName.Filter:= rStrODConfigNameFilter1;
  ODConfigName.FileName:= '';
  if ODConfigName.Execute then
  begin
    if not LazFileUtils.FileExistsUTF8(ODConfigName.FileName) then
    begin
      CreateNewDatabase(ODConfigName.FileName,MemoInfo);
      ActualConverter := ODConfigName.FileName;
      DisconnectFromDatabase(MemoInfo);
    end
    else
    begin
      //MemoInfo.Append(rStractSearchConfigFileExist);
      ShowMessage(rStractSearchConfigFileExist);
    end;
  end;
  SetStatusPanel;
end;

procedure TForm1.actCreateNewDatabaseRaikaExecute(Sender: TObject);
begin
  DisconnectFromDatabase(MemoInfo);
  // Erzeugen einer neuen Konfiguration
  if ActualConverter <> '' then
    ODConfigName.InitialDir:=ExtractFilePath(ActualConverter)
  else
    ODConfigName.InitialDir:=ExtractFilePath(ParamStr(0));
  ODConfigName.DefaultExt:='.sq3';
  ODConfigName.Filter:= rStrODConfigNameFilter1;
  ODConfigName.FileName:= '';
  if ODConfigName.Execute then
  begin
    if not LazFileUtils.FileExistsUTF8(ODConfigName.FileName) then
    begin
      CreateNewDatabase(ODConfigName.FileName,MemoInfo);
      ActualConverter := ODConfigName.FileName;
      DisconnectFromDatabase(MemoInfo);
      // Create the new entries
      if not ConnectToDatabase(ActualConverter,MemoInfo) then
      begin
        MemoInfo.Append('Error in DB open');
        exit; //==>
      end;
      CreateNewRaika;
      DisconnectFromDatabase(MemoInfo);
      // End create the new entries
    end
    else
    begin
      MemoInfo.Append(rStractSearchConfigFileExist);
      ShowMessage(rStractSearchConfigFileExist);
    end;
  end;
  SetStatusPanel;
end;

procedure TForm1.actEditFilterExecute(Sender: TObject);
begin
  if not ConnectToDatabase(ActualConverter,MemoInfo) then
  begin
    MemoInfo.Append(rStrErrorDBEditFilter);
    exit; //==>
  end;
  if not DoEditFilter then MemoInfo.Append(rStrErrorEditFilter);
  DisconnectFromDatabase(MemoInfo);
end;

procedure TForm1.actEditHashExecute(Sender: TObject);
begin
  if not ConnectToDatabase(ActualConverter,MemoInfo) then
  begin
    MemoInfo.Append(rStrErrorDBEditHash);
    exit; //==>
  end;
  if not DoEditHash then MemoInfo.Append(rStrErrorEditHash);
  DisconnectFromDatabase(MemoInfo);
end;

procedure TForm1.actEditSearchExecute(Sender: TObject);
begin
  if not ConnectToDatabase(ActualConverter,MemoInfo) then
  begin
    MemoInfo.Append(rStrErrorDBEditSearch);
    exit; //==>
  end;
  if not DoEditSearch then MemoInfo.Append(rStrErrorEditSearch);
  DisconnectFromDatabase(MemoInfo);
end;

procedure TForm1.actLangENExecute(Sender: TObject);
begin
  LastLang := 'en';
  SelectLanguage(LastLang);
end;

procedure TForm1.actQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TForm1.actSearchConfigExecute(Sender: TObject);
begin
  // Laden der vorhandenen Konfiguration
  if ActualConverter <> '' then
    ODConfigName.InitialDir:=ExtractFilePath(ActualConverter)
  else
    ODConfigName.InitialDir:=ExtractFilePath(ParamStr(0));
  ODConfigName.DefaultExt:='.sq3';
  ODConfigName.Filter:= rStrODConfigNameFilter1;
  ODConfigName.FileName:= '';
  if ODConfigName.Execute then
  begin
    if LazFileUtils.FileExistsUTF8(ODConfigName.FileName) then
      ActualConverter := ODConfigName.FileName
    else
      ShowMessage(rStractSearchConfigNoFile);
  end;
  SetStatusPanel;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  SetStatusPanel;
end;

procedure TForm1.XMLConfigMainRestoreProperties(Sender: TObject);
begin
  LastLang := XMLConfigMain.DoReadString('Persistent','Language','de');
  ActualConverter := XMLConfigMain.DoReadString('Persistent','ActualConverter','');
end;

procedure TForm1.XMLConfigMainSaveProperties(Sender: TObject);
begin
  if LastLang <> '' then
    XMLConfigMain.DoWriteString('Persistent','Language',LastLang)
  else
    XMLConfigMain.DoWriteString('Persistent','Language','de');
  if ActualConverter <> '' then
    XMLConfigMain.DoWriteString('Persistent','ActualConverter',ActualConverter);
end;

procedure TForm1.SetStatusPanel;
var
  Dummy: String;
begin
  // STatusbar Texte und die Längen setzen
  StatusBar1.Panels.Items[0].Width:= Round(Canvas.TextWidth(LastLang) * 1.2);
  StatusBar1.Panels.Items[0].Text:=LastLang;
  StatusBar1.Panels.Items[1].Width:= Round(Canvas.TextWidth(
    rStrStatusBar1PanelsItems1Text) * 1.2);
  StatusBar1.Panels.Items[1].Text:=rStrStatusBar1PanelsItems1Text;
  if ActualConverter = '' then
    Dummy := rStrNoFileLoaded
  else
    Dummy := ActualConverter;
  StatusBar1.Panels.Items[2].Width:= Round(Canvas.TextWidth(Dummy) * 1.2);
  StatusBar1.Panels.Items[2].Text:=Dummy;
end;

procedure TForm1.SetActualConverter(AValue: string);
begin
  if FActualConverter=AValue then Exit;
  FActualConverter:=AValue;
end;

procedure TForm1.DoConfigSet;
begin
  // Laden der Grundkonfiguration beim Start
  BaseFileName := ExtractFilePath(ParamStr(0)) + ExtractFileName(ParamStr(0));
  AppFileCfgName := LazFileUtils.GetAppConfigFileUTF8(False, False, True);
  XMLConfigMain.FileName := AppFileCfgName;
  XMLConfigMain.Restore;
end;

procedure TForm1.PrepareTranslation;
begin

end;

procedure TForm1.SelectLanguage(ALang: String);
begin
  // Edit controls auslesen
  PrepareTranslation;

  // Sprache umschalten
  SetDefaultLang(ALang);

  // FormatSettings umschalten
  UpdateFormatSettings(ALang);

  // BiDiMode anpassen (hier egal, wichtig für hebräische etc.)
  UpdateBiDiMode(ALang);

  // Spezielle Übersetzungsprozedur der Fenster aufrufen
  UpdateTranslation(ALang);

  // Neue Sprache im Menü markieren
  actLangDE.Enabled:= LastLang<>'de';
  actLangEN.Enabled:= LastLang<>'en';
  // Texte richtig setzen
  StatusBar1.Panels.Items[0].Text:= LastLang;
  Caption:= rStrMainCaption;
  // Texte für die Actions (Buttons etc.)
  actQuit.Caption:= rStractQuitCaption;
  actLangDE.Caption:= rStractLangDECaption;
  actLangEN.Caption:= rStractLangENCaption;
  actGetFilename.Caption:= rStractGetFilenameCaption;
  actSearchConfig.Caption:= rStractSearchConfigCaption;
  actEditSearch.Caption:=rStractEditSearchCaption;
  actEditHash.Caption:=rStractEditHashCaption;
  actEditFilter.Caption:=rStractEditFilterCaption;
  actCreateNewDatabase.Caption:= rStractCreateNewDatabase;
  // Hauptmenü Texte
  mnuSubFile.Caption:= rStrmnuSubFileCaption;
  mnuFileQuit.Caption:= rStractQuitCaption;
  mnuSubSprache.Caption:= rStrmnuSubSpracheCaption;
  mnuSubConfiguration.Caption:= rStrmnuSubConfiguration;
  //
  //// Button und Labels Text
  //BuGetFileName.Caption:= rStractGetFilenameCaption; // Sonst ist bei Standard der Button nicht richtig
  //LblFileName.Caption:= rStrLblFileNameCaption;
  //
  //// Dialoge
  //ODFileName.Title:= rStrODFileNameTitle;

  // STatusbar Texte und die Längen setzen
  SetStatusPanel;

end;

procedure TForm1.UpdateTranslation(ALang: String);
begin
  inherited UpdateTranslation(ALang);
end;

end.

