unit uFilter;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils
  , BOM_Filter, Forms, tiObject
  , BOM_Search, FACTORY_Search
  ;

type

TOnCreateNewSearchElement = function(aData:TSearch; Zeile: string):TModalResult;


{ TFilterConvertNumber
   InputFields  : Numbers of the Inputfields
   Filter       : FilterConvertNumber
   OutputFields : Numbers of the Outputfields, the same count as Inputfields
   Params       : Formatsettings for Input and Output
                    The Seperators must be quoted with "
                    and seperated by comma
                    Format:
                      Source.DecimalSeparator,
                      Source.ThousandSeparator,
                      Destination.DecimalSeparator,
                      Destination.ThousandSeparator
                    Eg: ",",".",".",","
}
TFilterConvertNumber = class(TFilter)
  public
    function CheckParams: Boolean; override;
    function Execute(InListe: TStrings; OutListe: TStrings): Boolean; override;
  end;

{ TFilterHash
   InputFields  : Must be empty
   Filter       : FilterHash
   OutputFields : Numbers of the Outputfields
   Params       : Value(s), if more then commaseperated
}
TFilterHash = class(TFilter)
private
  FHash: string;
  procedure SetHash(AValue: string);
  public
    function CheckParams: Boolean; override;
    function Execute(InListe: TStrings; OutListe: TStrings): Boolean; override;
    property Hash : string read FHash write SetHash;
  end;


{ TFilterInput
   InputFields  : Must be empty
   Filter       : FilterInput
   OutputFields : Numbers of the Outputfields
   Params       : Value(s), if more then commaseperated
}
TFilterInput = class(TFilter)
  public
    function CheckParams: Boolean; override;
    function Execute(InListe: TStrings; OutListe: TStrings): Boolean; override;
  end;

{ TFilterCopy
   InputFields  : Numbers of the Inputfields
   Filter       : FilterCopy
   OutputFields : Numbers of the Outputfields, the same count as Inputfields
   Params       : No needed
}
TFilterCopy = class(TFilter)
  public
    function CheckParams: Boolean; override;
    function Execute(InListe: TStrings; OutListe: TStrings): Boolean; override;
  end;

{ TFilterSearch
   InputFields  : Number of the Inputfield, only one allowed
   Filter       : FilterSearch
   OutputFields : Numbers of the Outputfields, depends on the 'Search'
   Params       : ClassName of the search eg. 'Search'
}
TFilterSearch = class(TFilter)
  private
    FOnCreateNewSearchElement: TOnCreateNewSearchElement;
    FSearchList: TSearchList;
    function DoShowEditSearchDetail(aData:TSearch; Zeile: string):TModalResult;
    procedure SetOnCreateNewSearchElement(AValue: TOnCreateNewSearchElement);
    procedure SetSearchList(AValue: TSearchList);
  public
    constructor Create; override;
    function CheckParams: Boolean; override;
    function Execute(InListe: TStrings; OutListe: TStrings): Boolean; override;
    property SearchList : TSearchList read FSearchList write SetSearchList;
    property OnCreateNewSearchElement : TOnCreateNewSearchElement read FOnCreateNewSearchElement write SetOnCreateNewSearchElement;
  end;


implementation
uses
  controls
  , regexpr
  , uListManager
  , ufrmeditsearchdetail
  , FACTORY_Filter
  ;


{ TFilterSearch }

function TFilterSearch.CheckParams: Boolean;
var
  aInList, aOutList : TStringList;
  aParamList : TStringList;
  Error: word;
  Num : Integer;
  i: Integer;
begin
  Result := False;
  // Check if fields are not empty
  if (InputFields = '') or (OutputFields = '')
     or (Filter = '') or (Params = '') then
    exit; //==>
  // Check if Fields are available and have the same length
  aInList := TStringList.Create;
  aOutList := TStringList.Create;
  aParamList := TStringList.Create;
  try
    aInList.CommaText:= InputFields;
    aOutList.CommaText:= OutputFields;
    aParamList.CommaText:= Params;
    if (aInList.Count <> 1) or (aOutList.Count <> 3)
       or (aParamList.Count <> 1) then exit; //==>
    // Check the content of the Input and Output
    for i := 0 to aInList.Count -1 do
    begin
      Val(aInList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
  finally
    aParamList.Free;
    aOutList.Free;
    aInList.Free;
  end;
  // default
  Result := True;
end;

constructor TFilterSearch.Create;
begin
  inherited Create;
  FSearchList := nil;
  FOnCreateNewSearchElement := @ShowEditSearchDetail;
end;

function TFilterSearch.Execute(InListe: TStrings; OutListe: TStrings): Boolean;
var
  aInList: TStringList;
  aOutList: TStringList;
  aParamList : TStringList;
  i: Integer;
  posI: Integer;
  posO: Integer;
  Error: word;
  strTemp : string;
  aInStr : String;
  RegexFound: Boolean;
  aValidObject : Boolean;
  aObject: TtiObject;
  mrResult: TModalResult;
  RegexObj: TRegExpr;
  aSearch : TSearch;
  DummyStr: String;
  iResultText, iCategory, iSubCategory : Integer;
begin
  Result := inherited;
  // Do nothing convert only copying
  aInList := TStringList.Create;
  aOutList := TStringList.Create;
  aParamList := TStringList.Create;
  try
    aInList.CommaText:= InputFields;
    aOutList.CommaText:= OutputFields;
    aParamList.CommaText:= Params;
    if (aInList.Count <> 1) or (aOutList.Count <> 3)
       or (aParamList.Count <> 1) then
      exit; //==>
    // check what is the max In or Out value
    // Copy the content
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], posO, Error);
      if (Error <> 0) or (posO < 0) then exit; //==>
      // if Liste is to short, make it longer
      while OutListe.Count < (posO + 1) do OutListe.Add('');
      ///
    end;
    if not assigned(FSearchList) then
       FSearchList := gListManager.SearchList;
    for i := 0 to aInList.Count -1 do
    begin
      Val(aInList.Strings[i], posI, Error);
      if (Error <> 0) or (posI < 0) or ((InListe.count-1) < posI)then exit; //==>
      // Search for the entry
      // Get the actual entries for better debugging
      aInStr:= InListe[posI];
      FSearchList.Clear;
      FSearchList.Read;
      //
      RegexFound:=false;
      RegexObj := TRegExpr.Create;
      try
        repeat
          aValidObject:=false;
          for aObject in FSearchList do
          begin
            aValidObject:=true;
            // build the regex-term
            strTemp := '.*'+ TSearch(aObject).SearchText +'.*';
            if strTemp = '.*.*' then continue; // => next term
            // start the regex search
            RegexObj.Expression:=strTemp;
            if RegexObj.Exec(aInStr) then
            begin
              // something found, transfer to the output and start the next input
              DummyStr := aOutList.Strings[0];
              Val(DummyStr, iResultText, Error);
              if (Error <> 0) or (iResultText < 0) then exit; //==>
              OutListe[iResultText] := TSearch(aObject).ResultText;

              DummyStr := aOutList.Strings[1];
              Val(DummyStr, iCategory, Error);
              if (Error <> 0) or (iCategory < 0) then exit; //==>
              OutListe[iCategory] := TSearch(aObject).Category;

              DummyStr := aOutList.Strings[2];
              Val(DummyStr, iSubCategory, Error);
              if (Error <> 0) or (iSubCategory < 0) then exit; //==>
              OutListe[iSubCategory] := TSearch(aObject).SubCategory;

              RegexFound:=true;
              break;
            end;
          end;
          // The whole list was searched
          if (not RegexFound) or (not aValidObject) then
          begin
            aSearch := gSearchFactory.CreateInstance('Search');
            mrResult := DoShowEditSearchDetail(aSearch,aInStr);
            if mrResult = mrOk then
            begin
              aSearch.Dirty:=true;
              FSearchList.Add(aSearch);
              FSearchList.Save;
              continue;
            end
            else
            begin
              aSearch.Free;
              if mrResult = mrAbort then
                exit
              else
              begin
                RegexFound:=true;
                break;
              end;
            end;
          end;
        until RegexFound;
      finally
        RegexObj.Free;
      end;
    end;
    Result := true;
  finally
    aParamList.Free;
    aOutList.Free;
    aInList.Free;
  end;
end;

procedure TFilterSearch.SetSearchList(AValue: TSearchList);
begin
  if FSearchList=AValue then Exit;
  FSearchList:=AValue;
end;

function TFilterSearch.DoShowEditSearchDetail(aData: TSearch; Zeile: string
  ): TModalResult;
begin
  Result := mrNone;
  if Assigned(FOnCreateNewSearchElement) then
    Result := FOnCreateNewSearchElement(aData,Zeile);
end;

procedure TFilterSearch.SetOnCreateNewSearchElement(
  AValue: TOnCreateNewSearchElement);
begin
  if FOnCreateNewSearchElement=AValue then Exit;
  FOnCreateNewSearchElement:=AValue;
end;

{ TFilterHash }

procedure TFilterHash.SetHash(AValue: string);
begin
  if FHash=AValue then Exit;
  FHash:=AValue;
end;

function TFilterHash.CheckParams: Boolean;
var
  aOutList : TStringList;
  Error: word;
  Num : Integer;
  i: Integer;
begin
  Result := False;
  // Check if fields are not empty
  if (OutputFields = '') or (Filter = '') then
    exit; //==>
  // Check if Fields are available and have the same length
  aOutList := TStringList.Create;
  try
    aOutList.CommaText:= OutputFields;
    if (aOutList.Count <= 0) then exit; //==>
    // Check the content of the OutputList
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
  finally
    aOutList.Free;
  end;
  // default
  Result := True;
end;

function TFilterHash.Execute(InListe: TStrings; OutListe: TStrings): Boolean;
var
  aOutList: TStringList;
  i: Integer;
  posO: Integer;
  Error: word;
begin
  Result := inherited;
  // Do nothing convert only copying
  aOutList := TStringList.Create;
  try
    aOutList.CommaText:= OutputFields;
    if (aOutList.Count <= 0) then
      exit; //==>
    // check what is the max In or Out value
    // Copy the content
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], posO, Error);
      if (Error <> 0) or (posO < 0) then exit; //==>
      // if Liste is to short, make it longer
      while OutListe.Count < (posO + 1) do OutListe.Add('');
      OutListe[posO] := Params;
    end;
    Result := true;
  finally
    aOutList.Free;
  end;
end;

{ TFilterConvertNumber }

function TFilterConvertNumber.CheckParams: Boolean;
var
  aInList, aOutList : TStringList;
  aParamList : TStringList;
  Error: word;
  Num : Integer;
  i: Integer;
begin
  Result := False;
  // Check if fields are not empty
  if (InputFields = '') or (OutputFields = '')
     or (Filter = '') or (Params = '') then
    exit; //==>
  // Check if Fields are available and have the same length
  aInList := TStringList.Create;
  aOutList := TStringList.Create;
  aParamList := TStringList.Create;
  try
    aInList.CommaText:= InputFields;
    aOutList.CommaText:= OutputFields;
    aParamList.CommaText:= Params;
    if (aInList.Count <= 0) or (aOutList.Count <= 0)
       or (aInList.Count <> aOutList.Count)
       or (aParamList.Count <> 4) then exit; //==>
    // Check the content of the Input and Output
    for i := 0 to aInList.Count -1 do
    begin
      Val(aInList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
  finally
    aParamList.Free;
    aOutList.Free;
    aInList.Free;
  end;
  // default
  Result := True;
end;

function TFilterConvertNumber.Execute(InListe: TStrings; OutListe: TStrings
  ): Boolean;
  function StrFormat2StrFormat(SourceStr: string; SourceFormat: TFormatSettings;
                               out DestStr: string; DestFormat: TFormatSettings):boolean;
  var
      Temp: Extended;
  begin
    Result:= false;
    if SourceStr = '' then
      exit; //==>
    try
      if not TryStrToFloat(SourceStr, Temp, SourceFormat) then
        exit; //==>
      DestStr:=FloatToStr(Temp,DestFormat);
      Result := true;
    except
      //
    end;
  end;
var
  aInList: TStringList;
  aOutList: TStringList;
  aParamList : TStringList;
  i: Integer;
  posI: Integer;
  posO: Integer;
  Error: word;
  SourceFSettings,DestFSettings: TFormatSettings;
  strTemp : string;
begin
  Result := inherited;
  // Do nothing convert only copying
  aInList := TStringList.Create;
  aOutList := TStringList.Create;
  aParamList := TStringList.Create;
  try
    aInList.CommaText:= InputFields;
    aOutList.CommaText:= OutputFields;
    aParamList.CommaText:= Params;
    if (aInList.Count <= 0) or (aOutList.Count <= 0)
       or (aInList.Count <> aOutList.Count)
       or (aParamList.Count <> 4) then
      exit; //==>
    // check what is the max In or Out value
    // Copy the content
    for i := 0 to aInList.Count -1 do
    begin
      Val(aInList.Strings[i], posI, Error);
      if (Error <> 0) or (posI < 0) or ((InListe.count-1) < posI)then exit; //==>
      Val(aOutList.Strings[i], posO, Error);
      if (Error <> 0) or (posO < 0) then exit; //==>
      // if Liste is to short, make it longer
      while OutListe.Count < (posO + 1) do OutListe.Add('');
      // Convert fehlt noch !!!!!
      SourceFSettings:=FormatSettings;
      SourceFSettings.DecimalSeparator:=aParamList[0][1];
      SourceFSettings.ThousandSeparator:=aParamList[1][1];
      DestFSettings:=FormatSettings;
      DestFSettings.DecimalSeparator:=aParamList[2][1];
      DestFSettings.ThousandSeparator:=aParamList[3][1];
      if not StrFormat2StrFormat(InListe[posI],SourceFSettings,strTemp,DestFSettings) then exit; //==>
      OutListe[posO] := strTemp;
     end;
    Result := true;
  finally
    aParamList.Free;
    aOutList.Free;
    aInList.Free;
  end;
end;

{ TFilterInput }

function TFilterInput.CheckParams: Boolean;
var
  aOutList : TStringList;
  Error: word;
  Num : Integer;
  i: Integer;
begin
  Result := False;
  // Check if fields are correct set
  if (InputFields <> '') or (OutputFields = '') or (Filter = '') then
    exit; //==>
  aOutList := TStringList.Create;
  try
    aOutList.CommaText:= OutputFields;
    if (aOutList.Count <= 0) then exit; //==>
    // Check the content of the Input and Output
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
  finally
    aOutList.Free;
  end;
  Result := True;
end;

function TFilterInput.Execute(InListe: TStrings; OutListe: TStrings): Boolean;
var
  aOutList: TStringList;
  i: Integer;
  posO: Integer;
  Error: word;
begin
  Result := inherited;
  // Do nothing convert only copying
  aOutList := TStringList.Create;
  try
    aOutList.CommaText:= OutputFields;
    if (aOutList.Count <= 0) then
      exit; //==>
    // check what is the max In or Out value
    // Copy the content
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], posO, Error);
      if (Error <> 0) or (posO < 0) then exit; //==>
      // if Liste is to short, make it longer
      while OutListe.Count < (posO + 1) do OutListe.Add('');
      OutListe[posO] := Params;
    end;
    Result := true;
  finally
    aOutList.Free;
  end;
end;

{ TFilterCopy }

function TFilterCopy.CheckParams: Boolean;
var
  aInList, aOutList : TStringList;
  Error: word;
  Num : Integer;
  i: Integer;
begin
  Result := False;
  // Check if fields are not empty
  if (InputFields = '') or (OutputFields = '') or (Filter = '') then
    exit; //==>
  // Check if Fields are available and have the same length
  aInList := TStringList.Create;
  aOutList := TStringList.Create;
  try
    aInList.CommaText:= InputFields;
    aOutList.CommaText:= OutputFields;
    if (aInList.Count <= 0) or (aOutList.Count <= 0)
       or (aInList.Count <> aOutList.Count) then exit; //==>
    // Check the content of the Input and Output
    for i := 0 to aInList.Count -1 do
    begin
      Val(aInList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
    for i := 0 to aOutList.Count -1 do
    begin
      Val(aOutList.Strings[i], Num, Error);
      if (Error <> 0) or (Num < 0) then exit; //==>
    end;
  finally
    aOutList.Free;
    aInList.Free;
  end;
  // default
  Result := True;
end;

function TFilterCopy.Execute(InListe: TStrings; OutListe: TStrings): Boolean;
var
  aInList: TStringList;
  aOutList: TStringList;
  i: Integer;
  posI: Integer;
  posO: Integer;
  Error: word;
begin
  Result := inherited;
  // Do nothing convert only copying
  aInList := TStringList.Create;
  aOutList := TStringList.Create;
  try
    aInList.CommaText:= InputFields;
    aOutList.CommaText:= OutputFields;
    if (aInList.Count <= 0) or (aOutList.Count <= 0)
       or (aInList.Count <> aOutList.Count) then
      exit; //==>
    // check what is the max In or Out value
    // Copy the content
    for i := 0 to aInList.Count -1 do
    begin
      Val(aInList.Strings[i], posI, Error);
      if (Error <> 0) or (posI < 0) or ((InListe.count-1) < posI)then exit; //==>
      Val(aOutList.Strings[i], posO, Error);
      if (Error <> 0) or (posO < 0) then exit; //==>
      // if Liste is to short, make it longer
      while OutListe.Count < (posO + 1) do OutListe.Add('');
      OutListe[posO] := InListe[posI];
     end;
    Result := true;
  finally
    aOutList.Free;
    aInList.Free;
  end;
end;


initialization

  gFilterFactory.RegisterClass('FilterCopy', TFilterCopy);
  gFilterFactory.RegisterClass('FilterInput', TFilterInput);
  gFilterFactory.RegisterClass('FilterHash', TFilterHash);
  gFilterFactory.RegisterClass('FilterConvertNumber', TFilterConvertNumber);
  gFilterFactory.RegisterClass('FilterSearch', TFilterSearch);


end.

