unit ufrmeditfilter;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  ActnList, Menus, Grids, Buttons, StdCtrls, BOM_Filter
  , tiModelMediator
  , FtiPerEditDialog
  , tiObject
  ;

type

  { TfrmEditFilter }

  TfrmEditFilter = class(TFormTIPerEditDialog)
    actAdd: TAction;
    actDel: TAction;
    actEdit: TAction;
    actImport: TAction;
    actSGFilterClick: TAction;
    ActionList1: TActionList;
    ImageList1: TImageList;
    sgFilter: TStringGrid;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    procedure actAddExecute(Sender: TObject);
    procedure actDelExecute(Sender: TObject);
    procedure actEditExecute(Sender: TObject);
    procedure actImportExecute(Sender: TObject);
    procedure actSGFilterClickExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sgFilterDblClick(Sender: TObject);
  private
    FFilters: TFilterList;
    FMediator: TtiModelMediator;
    procedure SetFilters(AValue: TFilterList);
    procedure SetupMediators;
    { private declarations }
  protected
   procedure SetData(const AValue: TtiObject); override;
   function  FormIsValid : boolean; override;
   function  FormIsEdited : boolean; override;
  public
    { public declarations }
    property Filters : TFilterList read FFilters write SetFilters;
    class function Execute(const AData : TtiObject; pReadOnly : boolean = false): boolean; reintroduce;
  end;


//function ShowEditFilter(Filters : TFilterList):Boolean;


implementation

uses
  uListManager
  , tiDialogs
  , tiListMediators
  , tiBaseMediator
  , tiMediators
  , ufrmeditfilterdetail
  ;

{$R *.lfm}

//function ShowEditFilter(Filters : TFilterList):Boolean;
//var
//  frm : TfrmEditFilter;
//begin
//  Result := False;
//  frm := TfrmEditFilter.Create(nil);
//  try
//    if not Assigned(Filters) then
//      exit; //==>
//    frm.Filters := Filters;
//    frm.ShowModal;
//    Result := True;
//  finally
//    frm.Free;
//  end;
//end;
//

{ TfrmEditFilter }

procedure TfrmEditFilter.actAddExecute(Sender: TObject);
var
  c: TFilter;
begin
  c := TFilter.CreateNew;
  if ShowEditFilterDetail(c) then
  begin
    TFilterList(Databuffer).Add(c);
    TFilterList(Databuffer).Save;
  end
  else
    c.Free;
  SetupMediators;
end;

procedure TfrmEditFilter.actDelExecute(Sender: TObject);
var
  c: TFilter;
begin
  if sgFilter.SelectedRangeCount < 1 then
  begin
    tiAppError('You need to select a Filter first');
    Exit;
  end;
  c := TFilter(TtiStringGridMediatorView(FMediator.FindByComponent(sgFilter).Mediator).SelectedObject);

  if tiAppConfirmation('Are you sure you want to delete <%s>', [c.Filter]) then
  begin
    { We can't use .Deleted property here, because we don't actually save
      changes. This means the ObjectState will only be posDelete and not
      posDeleted, which is what .FreeDeleted is looking for. }
//    c.Deleted := True;
    TFilterList(Databuffer).BeginUpdate('');
     if Assigned(c) then
     begin
       c.Deleted := True;
       { If we used a database, this call would trigger the delete visitor.
         The database referential integrity checks would also prevent the
         deletion of a record is use. }
       c.Save;
       { If the Save() was successful, this call would remove and free the
         object in memory. }
       TFilterList(Databuffer).Remove(c);
     end;
     TFilterList(Databuffer).EndUpdate;
  end;
  SetupMediators;
end;

procedure TfrmEditFilter.actEditExecute(Sender: TObject);
var
  c: TFilter;
begin
  if sgFilter.SelectedRangeCount < 1 then
  begin
    tiAppError('You need to select a Filter first');
    Exit;
  end;
  c := TFilter(TtiStringGridMediatorView(FMediator.FindByComponent(sgFilter).Mediator).SelectedObject);

  if not Assigned(c) then
    Exit; //==>

  if ShowEditFilterDetail(c) then
  begin
    c.ObjectState := posUpdate;
    c.Save;
  end;
  SetupMediators;
end;

procedure TfrmEditFilter.actImportExecute(Sender: TObject);
begin
  // Nothing to do
end;

procedure TfrmEditFilter.actSGFilterClickExecute(Sender: TObject);
begin
end;

procedure TfrmEditFilter.FormCreate(Sender: TObject);
begin
  inherited
end;

procedure TfrmEditFilter.sgFilterDblClick(Sender: TObject);
begin
  actEditExecute(Sender);
end;

procedure TfrmEditFilter.SetupMediators;
begin
  if not Assigned(FMediator) then
  begin
    FMediator := TtiModelMediator.Create(self);
    FMediator.AddComposite('InputFields;Filter;OutputFields;Params;Remark', sgFilter);
  end;
  FMediator.Subject := Databuffer;
  FMediator.Active := True;
end;

procedure TfrmEditFilter.SetData(const AValue: TtiObject);
begin
  inherited SetData(AValue);
  if AValue is TFilterList then
  begin
    SetupMediators;
  end;
end;

function TfrmEditFilter.FormIsValid: boolean;
begin
  Result:=inherited FormIsValid;
end;

function TfrmEditFilter.FormIsEdited: boolean;
begin
  Result:=inherited FormIsEdited;
end;

class function TfrmEditFilter.Execute(const AData: TtiObject;
  pReadOnly: boolean): boolean;
var
  lForm : TfrmEditFilter;
begin
  lForm := Create(nil);
  try
    lForm.Data := AData;
    if pReadOnly then
      lForm.RO.Enabled:= true;
    lForm.RO.ReadOnly := pReadOnly;
    result := lForm.ShowModal = mrOK;
  finally
    lForm.Free;
  end;
end;

procedure TfrmEditFilter.SetFilters(AValue: TFilterList);
begin
  if FFilters=AValue then Exit;
  FFilters:=AValue;
  SetupMediators;
end;


initialization
  RegisterFallBackMediators;
  gMediatorManager.RegisterMediator(TtiStringGridMediatorView, TFilterList);
end.

