unit ufrmeditHash;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  ActnList, Menus, Grids, Buttons, StdCtrls
  , BOM_Hash
  , VIEW_Hash
  , tiModelMediator
  , FtiPerEditDialog
  , tiObject
  ;

type

  { TfrmEditHash }

  TfrmEditHash = class(TFormTIPerEditDialog)
    actAdd: TAction;
    actDel: TAction;
    actEdit: TAction;
    actImport: TAction;
    ActionList1: TActionList;
    BitBtn1: TBitBtn;
    ImageList1: TImageList;
    OpenDialog1: TOpenDialog;
    sgHash: TStringGrid;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    procedure actImportExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure actAddExecute(Sender: TObject);
    procedure actDelExecute(Sender: TObject);
    procedure actEditExecute(Sender: TObject);
    procedure sgHashDblClick(Sender: TObject);
  private
    FMediator: TtiModelMediator;
    FDisplayList : THashDisplayList;
    procedure SetupMediators;
    { private declarations }
  protected
    procedure SetData(const AValue: TtiObject); override;
    function  FormIsValid : boolean; override;
    function  FormIsEdited : boolean; override;
  public
    { public declarations }
    Property DisplayList : THashDisplayList Read FDisplayList Write FDisplayList;
  end;


implementation

uses
  uListManager
  , tiDialogs
  , tiListMediators
  , tiBaseMediator
  , tiMediators
  , LazFileUtils
  ;

{$R *.lfm}


{ TfrmEditHash }

procedure TfrmEditHash.FormCreate(Sender: TObject);
begin
  inherited;
end;

procedure TfrmEditHash.actImportExecute(Sender: TObject);
var
  FileName: String;
  aData : THash;
  aList : THashList;
  aHash : String;
  i, Anz, Akt: Integer;
  SGTemp : TStringGrid;
begin
  aList := THashList(Databuffer);
  if aList.Count > 0 then
  begin
    if QuestionDlg('Import Hash',
                   'Daten vorhanden'+#13+'Sollen die Daten trotzdem importiert werden ?',
                   mtConfirmation,
                   [mrYes, 'Ja', mrNo, 'Nein'], 0) = mrNo then
    exit;
  end;


  OpenDialog1.InitialDir:=ExtractFilePath(ParamStr(0));
  OpenDialog1.DefaultExt:='.txt';
  OpenDialog1.Filter:= 'TextFiles|*.txt';
  OpenDialog1.FileName:= '*SGHash.txt';
  if OpenDialog1.Execute then
  begin
    if not LazFileUtils.FileExistsUTF8(OpenDialog1.FileName) then
    begin
      ShowMessage('File does not exist');
      exit;
    end
  end;
  FileName:=OpenDialog1.FileName;
  SGTemp := TStringGrid.Create(self);
  try
    if FileExists(FileName) then
      SGTemp.LoadFromCSVFile(FileName);
    if SGTemp.ColCount <> 3 then
    begin
      ShowMessage( 'Fileformat not valid - expected 3 colums, found ' + IntToStr(SGTemp.RowCount));
      exit;
    end;
    anz := 0;
    akt := 0;
    for i := 0 to (SGTemp.RowCount-1) do
    begin
      inc(anz);
      aHash:=SGTemp.Cells[1,i];
      if aList.FindHash(aHash) <> 0 then
         continue;
      inc(akt);
      aData := THash.CreateNew;
      aData.FileHash:= aHash;
      aData.FileName:=SGTemp.Cells[2,i];
      aData.ObjectState  := posCreate;
      aData.Dirty:=true;
      aData.Save;
    end;
    aList.Clear;
    aList.Read;
    showmessage('Read ' + intToStr(anz) + ' and used '+ IntToStr(akt)+ ' entries');
  finally
    SGTemp.Free;
  end;
end;

procedure TfrmEditHash.FormDestroy(Sender: TObject);
begin
  inherited;
end;

procedure TfrmEditHash.actAddExecute(Sender: TObject);
begin
  ShowMessage('Not implemented');
end;

procedure TfrmEditHash.actDelExecute(Sender: TObject);
var
  c: THash;
  D : THashDisplay;
  M : TtiMediatorView;
begin
  if sgHash.SelectedRangeCount < 1 then
  begin
    tiAppError('You need to select a Hash first');
    Exit;
  end;
  M:= FMediator.FindByComponent(sgHash).Mediator;
  D := THashDisplay(TtiStringGridMediatorView(M).SelectedObject);

  c := D.AHash;

  if tiAppConfirmation('Are you sure you want to delete <%s>', [c.FileHash]) then
  begin
    { We can't use .Deleted property here, because we don't actually save
      changes. This means the ObjectState will only be posDelete and not
      posDeleted, which is what .FreeDeleted is looking for. }
    THashList(Databuffer).BeginUpdate('');
    if Assigned(c) then
    begin
      c.Deleted := True;
      { If we used a database, this call would trigger the delete visitor.
        The database referential integrity checks would also prevent the
        deletion of a record is use. }
      c.Save;
      { If the Save() was successful, this call would remove and free the
        object in memory. }
      THashList(Databuffer).Remove(c);
    end;
    THashList(Databuffer).EndUpdate;
  end;
//  SetupMediators;
end;

procedure TfrmEditHash.actEditExecute(Sender: TObject);
begin
  ShowMessage('Not implemented');
end;

procedure TfrmEditHash.sgHashDblClick(Sender: TObject);
begin
  actDelExecute(Sender);
end;

procedure TfrmEditHash.SetupMediators;
begin
  if not Assigned(FMediator) then
  begin
    FMediator := TtiModelMediator.Create(self);
    FMediator.AddComposite('StoreDate;FileHash;FileName', sgHash);
  end;
  FMediator.Subject := DisplayList;
  FMediator.Active := True;
end;

procedure TfrmEditHash.SetData(const AValue: TtiObject);
begin
  inherited SetData(AValue);
  if AValue is THashList then
  begin
    DisplayList := THashDisplayList.CreateCustom(Databuffer as TtiObjectList);
    // The next call is important, because we have first created the data and after
    //   this the Display List, so we have to inform the displaylist for the data
    DisplayList.Update(Databuffer as TtiObjectList, noChanged, nil);
    SetupMediators;
  end;
end;

function TfrmEditHash.FormIsValid: boolean;
begin
  Result:=inherited FormIsValid;
end;

function TfrmEditHash.FormIsEdited: boolean;
begin
  Result:=inherited FormIsEdited;
end;


initialization
  RegisterFallBackMediators;
  gMediatorManager.RegisterMediator(TtiStringGridMediatorView, THashDisplayList);
end.

