unit ufrmeditsearch;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  ActnList, Menus, Grids, Buttons, StdCtrls, BOM_Search
  , tiModelMediator
  , FtiPerEditDialog
  , tiObject
  ;

type

  { TfrmEditSearch }

  TfrmEditSearch = class(TFormTIPerEditDialog)
    actAdd: TAction;
    actDel: TAction;
    actEdit: TAction;
    actImport: TAction;
    actSGSearchClick: TAction;
    ActionList1: TActionList;
    BitBtn1: TBitBtn;
    ImageList1: TImageList;
    OpenDialog1: TOpenDialog;
    sgSearch: TStringGrid;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    procedure actAddExecute(Sender: TObject);
    procedure actDelExecute(Sender: TObject);
    procedure actEditExecute(Sender: TObject);
    procedure actImportExecute(Sender: TObject);
    procedure actSGSearchClickExecute(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sgSearchDblClick(Sender: TObject);
  private
    FSearch: TSearchList;
    FMediator: TtiModelMediator;
    FZeile: string;
    procedure SetSearch(AValue: TSearchList);
    procedure SetupMediators;
    procedure SetZeile(AValue: string);
    { private declarations }
   protected
    procedure SetData(const AValue: TtiObject); override;
    function  FormIsValid : boolean; override;
    function  FormIsEdited : boolean; override;
  public
    { public declarations }
    property Search : TSearchList read FSearch write SetSearch;
    property Zeile : string read FZeile write SetZeile;
    class function Execute(const AData : TtiObject; AZeile: string; pReadOnly : boolean = false): boolean; reintroduce;
  end;


//function ShowEditSearch(Search : TSearchList; Zeile: string):TModalResult;


implementation

uses
  uListManager
  , tiDialogs
  , tiListMediators
  , tiBaseMediator
  , tiMediators
  , ufrmeditSearchdetail
  , LazFileUtils
  ;

{$R *.lfm}

//function ShowEditSearch(Search : TSearchList; Zeile: string):TModalResult;
//var
//  frm : TfrmEditSearch;
//begin
//  Result := mrNone;
//  if not Assigned(Search) then
//    exit; //==>
//  frm := TfrmEditSearch.Create(nil);
//  try
//    frm.Search := Search;
//    frm.Zeile:=Zeile;
//    Result := frm.ShowModal;
//  finally
//    frm.Free;
//  end;
//end;


{ TfrmEditSearch }

procedure TfrmEditSearch.actAddExecute(Sender: TObject);
var
  c: TSearch;
begin
  c := TSearch.CreateNew;
  if ShowEditSearchDetail(c,Zeile)=mrOK then
  begin
    TSearchList(Databuffer).Add(c);
    TSearchList(Databuffer).Save;
  end
  else
    c.Free;
  SetupMediators;
end;

procedure TfrmEditSearch.actDelExecute(Sender: TObject);
var
  c: TSearch;
begin
  if sgSearch.SelectedRangeCount < 1 then
  begin
    tiAppError('You need to select a Search first');
    Exit;
  end;
  c := TSearch(TtiStringGridMediatorView(FMediator.FindByComponent(sgSearch).Mediator).SelectedObject);

  if tiAppConfirmation('Are you sure you want to delete <%s>', [c.SearchText]) then
  begin
    { We can't use .Deleted property here, because we don't actually save
      changes. This means the ObjectState will only be posDelete and not
      posDeleted, which is what .FreeDeleted is looking for. }
//    c.Deleted := True;
    TSearchList(Databuffer).BeginUpdate('');
     if Assigned(c) then
     begin
       c.Deleted := True;
       { If we used a database, this call would trigger the delete visitor.
         The database referential integrity checks would also prevent the
         deletion of a record is use. }
       c.Save;
       { If the Save() was successful, this call would remove and free the
         object in memory. }
       TSearchList(Databuffer).Remove(c);
     end;
     TSearchList(Databuffer).EndUpdate;
  end;
end;

procedure TfrmEditSearch.actEditExecute(Sender: TObject);
var
  c: TSearch;
begin
  if sgSearch.SelectedRangeCount < 1 then
  begin
    tiAppError('You need to select a Search first');
    Exit;
  end;
  c := TSearch(TtiStringGridMediatorView(FMediator.FindByComponent(sgSearch).Mediator).SelectedObject);

  if not Assigned(c) then
    Exit; //==>

  if ShowEditSearchDetail(c,Zeile)=mrOK then
  begin
    c.ObjectState := posUpdate;
    c.Save;
  end;
  SetupMediators;
end;

procedure TfrmEditSearch.actImportExecute(Sender: TObject);
var
  FileName: String;
  aEntry : String;
  aData : TSearch;
  aList : TSearchList;
  i,j, Anz, Akt: Integer;
  SGTemp : TStringGrid;
  found: Boolean;
begin
  aList := TSearchList(Databuffer);
  if aList.Count > 0 then
  begin
    if QuestionDlg('Import Search',
                   'Daten vorhanden'+#13+'Sollen die Daten trotzdem importiert werden ?',
                   mtConfirmation,
                   [mrYes, 'Ja', mrNo, 'Nein'], 0) = mrNo then
    exit;
  end;


    OpenDialog1.InitialDir:=ExtractFilePath(ParamStr(0));
    OpenDialog1.DefaultExt:='.txt';
    OpenDialog1.Filter:= 'TextFiles|*.txt';
    OpenDialog1.FileName:= '*SGSearch.txt';
    if OpenDialog1.Execute then
    begin
      if not LazFileUtils.FileExistsUTF8(OpenDialog1.FileName) then
      begin
        ShowMessage('File does not exist');
        exit;
      end
    end;
    FileName:=OpenDialog1.FileName;
    SGTemp := TStringGrid.Create(self);
    anz := 0;
    akt := 0;
    try
      if FileExists(FileName) then
        SGTemp.LoadFromCSVFile(FileName);
      if SGTemp.ColCount <> 5 then
      begin
        ShowMessage( 'Fileformat not valid - expected 5 colums, found ' + IntToStr(SGTemp.RowCount));
        exit;
      end;
      for i := 0 to (SGTemp.RowCount-1) do
      begin
        inc(anz);
        aEntry := SGTemp.Cells[1,i];
        found := false;
        for j:= 0 to (aList.count-1) do
        begin
          if (aList.Items[j].SearchText = aEntry) then
          begin
             found := true;
             break;
          end;
        end;
        if found then
          continue; // ==>
        inc(akt);
        aData := TSearch.CreateNew;
        aData.SearchText:= aEntry;
        aData.ResultText:=SGTemp.Cells[2,i];
        aData.Category:=SGTemp.Cells[3,i];
        aData.SubCategory:=SGTemp.Cells[4,i];
        aData.ObjectState  := posCreate;
        aData.Dirty:=true;
        aData.Save;
        aList.Add(aData);
      end;
      showmessage('Read ' + intToStr(anz) + ' and used '+ IntToStr(akt)+ ' entries');
    finally
      SGTemp.Free;
    end;

 end;

procedure TfrmEditSearch.actSGSearchClickExecute(Sender: TObject);
begin
end;

procedure TfrmEditSearch.FormActivate(Sender: TObject);
begin
end;



procedure TfrmEditSearch.FormCreate(Sender: TObject);
begin
  Inherited;
end;

procedure TfrmEditSearch.sgSearchDblClick(Sender: TObject);
begin
  actEditExecute(Sender);
end;

procedure TfrmEditSearch.SetupMediators;
begin
  if not Assigned(FMediator) then
  begin
    FMediator := TtiModelMediator.Create(self);
    FMediator.AddComposite('SearchText;ResultText;Category;SubCategory;Remark', sgSearch);
  end;
  FMediator.Subject := Databuffer;
  FMediator.Active := True;
end;

procedure TfrmEditSearch.SetZeile(AValue: string);
begin
  if FZeile=AValue then Exit;
  FZeile:=AValue;
end;

procedure TfrmEditSearch.SetData(const AValue: TtiObject);
begin
  inherited SetData(AValue);
  if AValue is TSearchList then
  begin
    SetupMediators;
  end;
end;

function TfrmEditSearch.FormIsValid: boolean;
begin
  Result:=inherited FormIsValid;
end;

function TfrmEditSearch.FormIsEdited: boolean;
begin
  Result:=inherited FormIsEdited;
end;

class function TfrmEditSearch.Execute(const AData: TtiObject; AZeile: string;
  pReadOnly: boolean): boolean;
var
  lForm : TfrmEditSearch;
begin
  lForm := Create(nil);
  try
    lForm.Data := AData;
    lForm.Zeile:= AZeile;
    //if pReadOnly then
    //  lForm.RO.Enabled:= true;
    //lForm.RO.ReadOnly := pReadOnly;
    result := lForm.ShowModal = mrOK;
  finally
    lForm.Free;
  end;
end;


procedure TfrmEditSearch.SetSearch(AValue: TSearchList);
begin
  if FSearch=AValue then Exit;
  FSearch:=AValue;
  SetupMediators;
end;


initialization
  RegisterFallBackMediators;
  gMediatorManager.RegisterMediator(TtiStringGridMediatorView, TSearchList);
end.

