unit uListManager;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils
  ,tiObject
  ,BOM_Filter
  ,BOM_Search
  ,BOM_Hash
;

type

{ TListManager }

{ Holds all the persistent lists of the project, use only the
   singleton gListManager }
TListManager = class(TtiObject)
private
  FTFilterList: TFilterList;
  FTHashList: THashList;
  FTSearchList: TSearchList;
public
  constructor Create; override;
  destructor  Destroy; override;
published
  property FilterList: TFilterList read FTFilterList;
  property SearchList: TSearchList read FTSearchList;
  property HashList: THashList read FTHashList;
end;


{ Global singleton, holds all the persistent lists of the project }
function gListManager: TListManager;


implementation

var
  uManager: TListManager;

function gListManager: TListManager;
begin
  if not Assigned(uManager) then
    uManager:= TListManager.Create;
  result:= uManager;
end;

{ TListManager }

constructor TListManager.Create;
begin
  inherited Create;
  FTFilterList:= TFilterList.Create;
  FTFilterList.Owner := self;
  FTSearchList:=TSearchList.Create;
  FTSearchList.Owner := self;
  FTHashList:= THashList.Create;
  FTHashList.Owner:= self;
end;

destructor TListManager.Destroy;
begin
  FTHashList.Free;
  FTSearchList.Free;
  FTFilterList.Free;
  inherited Destroy;
end;


finalization
  uManager.Free;

end.

