unit uparser;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

Procedure ParseDelimitedText(const AValue: string; const ADelimiter, AQuoteChar: Char; TS: TStrings);
Function ReadTextFirstLines(const FileName: string; MaxLines: integer; TS: TStrings):boolean;
Function StrFormat2StrFormat(SourceStr: string; SourceFormat: TFormatSettings;
                             out DestStr: string; DestFormat: TFormatSettings):boolean;

implementation
uses UTextFileStream;

(*
aus Grids, ab Zeile 10490
*)
Procedure ParseDelimitedText(const AValue: string; const ADelimiter, AQuoteChar: Char; TS: TStrings);
{ Helper function for LoadFromCSVFile
  Adapted from TStrings.SetDelimitedText
  - Only ADelimiter is used for separating the fields and not other whitespace
  - If a field is quoted and it contains AQuoteChar, this occurrence is treated as a literal part of the field
  - As per RFC4180 whitespace is considered to be part of the field, even if the field is not quoted
  - Trailing spaces of a quoted field are trimmed
  Example with ADelimiter = ',' and AQuoteChar = '"'
  AValue = '111,2,22,333' -> 111|2|22|333
  AValue = '111,"2,22",333' -> 111|2,22|333
  AValue = '111,  222  ,333' -> 111|   222   |333
}
var i,j:integer;
    aNotFirst:boolean;
begin
  TS.BeginUpdate;
  i:=1;
  j:=1;
  aNotFirst:=false;
  try
    TS.Clear;
    while i<=length(AValue) do
    begin
      // skip delimiter
      if aNotFirst and (i<=length(AValue)) and (AValue[i]=ADelimiter) then inc(i);
      // read next string
      if i<=length(AValue) then
      begin
        if AValue[i]=AQuoteChar then
        begin
          // next string is quoted
          j:=i+1;
           while (j<=length(AValue)) and
                 ( (AValue[j]<>AQuoteChar) or
                 ( (j+1<=length(AValue)) and (AValue[j+1]=AQuoteChar) ) ) do
           begin
             if (j<=length(AValue)) and (AValue[j]=AQuoteChar) then
               inc(j,2)
             else
               inc(j);
           end;
           // j is position of closing quote
           TS.Add( StringReplace (Copy(AValue,i+1,j-i-1),
                           AQuoteChar+AQuoteChar,AQuoteChar, [rfReplaceAll]));
           i:=j+1;
        end
        else
        begin
          // next string is not quoted
          j:=i;
          while (j<=length(AValue)) and
                //basically any other character means some invalid text
                ((Ord(AValue[j])>=Ord(' ')) or (AValue[j] in [#10,#13,#9,#0])) and
                (AValue[j]<>ADelimiter) do inc(j);
          TS.Add(Copy(AValue,i,j-i));
          i:=j;
        end;
     end
     else
     begin
      if aNotFirst then TS.Add('');
     end;
     // skip trailing spaces of a quoted field
     // not really sure if that is RFC4180 compliant
     while (i<=length(AValue)) and (Ord(AValue[i])<=Ord(' ')) and (AValue[i] <> ADelimiter) do inc(i);
     aNotFirst:=true;
    end; //end of string
  finally
   TS.EndUpdate;
  end;
end;//ParseDelimitedText


function ReadTextFirstLines(const FileName: string; MaxLines: integer;
  TS: TStrings): boolean;
var
    TFS : TTextFileStream;
    i: integer;
begin
  Result:= false;
  if (not assigned(TS)) or (MaxLines < 1) or (FileName = '') then exit;
  TFS := TTextFileStream.Create(FileName,fmOpenRead);
  try
    i := 0;
    while (not TFS.Eof) and (i < MaxLines ) do
    begin
      TS.Append(TFS.ReadLn);
      inc(i);
    end;
    Result:= true;
  finally
    TFS.Free;
  end;
end;


function StrFormat2StrFormat(SourceStr: string; SourceFormat: TFormatSettings;
                             out DestStr: string; DestFormat: TFormatSettings):boolean;
var
    Temp: Extended;
begin
  if SourceStr = '' then
  begin
    Result:= false;
    exit;
  end;
  try
    TryStrToFloat(SourceStr, Temp, SourceFormat);
    DestStr:=FloatToStr(Temp,DestFormat);
  except
    Result:=false;
  end;
end;


(*
sysstr.inc

Function StrToFloatDef(Const S: String; Const Default: Extended; Const FormatSettings: TFormatSettings): Extended;

begin
   if not TextToFloat(PChar(pointer(S)),Result,fvExtended,FormatSettings) then
     Result:=Default;
end;

Function TextToFloat(Buffer: PChar; Out Value: Extended; Const FormatSettings: TFormatSettings): Boolean;

Var
  E,P : Integer;
  S : String;

Begin
  S:=StrPas(Buffer);
  //ThousandSeparator not allowed as by Delphi specs
  if (FormatSettings.ThousandSeparator <> FormatSettings.DecimalSeparator) and
     (Pos(FormatSettings.ThousandSeparator, S) <> 0) then
    begin
      Result := False;
      Exit;
    end;
  if (FormatSettings.DecimalSeparator <> '.') and
     (Pos('.', S) <>0) then
    begin
      Result := False;
      Exit;
    end;
  P:=Pos(FormatSettings.DecimalSeparator,S);
  If (P<>0) Then
    S[P] := '.';
  Val(trim(S),Value,E);
  Result:=(E=0);
End;


*)






(*

var
  FSettings: TFormatSettings;


begin
  FSettings := FormatSettings; //add this to get rid of potential garbage
  FSettings.DecimalSeparator:='.';
  lNumber := StrToFloat(Str, FSettings);

  //prevent unwanted re-setting of DefaultFormatsettings
   Application.UpdateFormatSettings := False;
   //Now it is safe to set this for the lifetime of my app
   DefaultFormatSetttings.DecimalSeparator := '.';










Procedure TStrings.SetDelimitedText(const AValue: string);
var i,j:integer;
    aNotFirst:boolean;
begin
 CheckSpecialChars;
 BeginUpdate;

 i:=1;
 j:=1;
 aNotFirst:=false;

 try
  Clear;
  If StrictDelimiter then
    begin
    // Easier, faster loop.
    While I<=Length(AValue) do
      begin
      If (AValue[I] in [FDelimiter,#0]) then
        begin
        Add(Copy(AValue,J,I-J));
        J:=I+1;
        end;
      Inc(i);
      end;
    If (Length(AValue)>0) then
      Add(Copy(AValue,J,I-J));
    end
  else
    begin
    while i<=length(AValue) do begin
     // skip delimiter
     if aNotFirst and (i<=length(AValue)) and (AValue[i]=FDelimiter) then inc(i);

     // skip spaces
     while (i<=length(AValue)) and (Ord(AValue[i])<=Ord(' ')) do inc(i);

     // read next string
     if i<=length(AValue) then begin
      if AValue[i]=FQuoteChar then begin
       // next string is quoted
       j:=i+1;
       while (j<=length(AValue)) and
             ( (AValue[j]<>FQuoteChar) or
               ( (j+1<=length(AValue)) and (AValue[j+1]=FQuoteChar) ) ) do begin
        if (j<=length(AValue)) and (AValue[j]=FQuoteChar) then inc(j,2)
                                                          else inc(j);
       end;
       // j is position of closing quote
       Add( StringReplace (Copy(AValue,i+1,j-i-1),
                           FQuoteChar+FQuoteChar,FQuoteChar, [rfReplaceAll]));
       i:=j+1;
      end else begin
       // next string is not quoted
       j:=i;
       while (j<=length(AValue)) and
             (Ord(AValue[j])>Ord(' ')) and
             (AValue[j]<>FDelimiter) do inc(j);
       Add( Copy(AValue,i,j-i));
       i:=j;
      end;
     end else begin
      if aNotFirst then Add('');
     end;

     // skip spaces
     while (i<=length(AValue)) and (Ord(AValue[i])<=Ord(' ')) do inc(i);

     aNotFirst:=true;
    end;
    end;
 finally
   EndUpdate;
 end;
end;



*)


end.

