unit testcheckingbomfilter_tst;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry
  , BOM_Filter
  , BOM_Hash
  , BOM_Search
  , uFilter
  , uListManager
  , tiOPFManager
  , tiLog
  ;

type

  { TTestCheckingBOM }

  TTestCheckingBOM= class(TTestCase)
  private
  protected
    procedure SetUp; override;
    procedure TearDown; override;
  public
    constructor CreateWithName(const AName: string); override;
  published
    procedure TestLookForTableFiltersNotInDB;
    procedure TestCreateTableFiltersInBOM;
    procedure TestFilterCRUD;

    procedure TestFilterCopyWithOkParams;
    procedure TestFilterInputWithOkParams;
    procedure TestFilterHashWithOkParams;
    procedure TestFilterConvertNumberWithOkParams;
    procedure TestFilterSearchWithOkParams;

  end;

implementation

uses
  Forms, Controls;

constructor TTestCheckingBOM.CreateWithName(const AName: string);
begin
  Inherited CreateWithName('WithDatabase.TestCheckingBOM');
end;

procedure TTestCheckingBOM.TestCreateTableFiltersInBOM;
begin
  Log([ClassName],lsDebug);
  BOM_Filter.CreateTables;
  Check(GTIOPFManager.TableExists('Filters'),'Table Filters not in Database');
end;


procedure TTestCheckingBOM.TestFilterCopyWithOkParams;
var
  aFilterList: TFilterList;
  aFilter, aTestFilter : TFilter;
  aClassname : string;
  aInStr : TStringList;
  aOutStr : TStringList;
begin
  Log([ClassName],lsDebug);
  BOM_Filter.CreateTables;
  Check(GTIOPFManager.TableExists('Filters'),'Table Filters not in Database');
  aFilterList := gListManager.FilterList;
  aInStr := TStringList.Create;
  aOutStr := TStringList.Create;
  try
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Filters are not empty');

    aFilter := TFilter(gFilterFactory.CreateInstance('FilterCopy'));
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterCopy', 'Filtercopy not correct created');
    CheckFalse(aFilter.CheckParams, 'Filtercopy CheckParams with no params isnt false' );
    aFilter.InputFields:= '0,1';
    aFilter.OutputFields:= '2,3';
    aFilter.Params:='none';
    CheckTrue(aFilter.CheckParams, 'Filtercopy CheckParams with correct filled in fields fails' );
    aInStr.Clear;
    aInStr.Add('F1');
    aInStr.Add('F2');
    aOutStr.Clear;
    CheckTrue(aFilter.Execute(aInStr,aOutStr), 'Filtercopy Execute with correct filled in fields fails' );
    Check(aOutStr.Count=4, 'Execute Outfield have not the correct size (should 4) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='', 'Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[2]='F1', 'Execute Outfield 2 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[3]='F2', 'Execute Outfield 3 false value (' + aOutStr[0] + '), should be empty');

    aFilterList.Add(aFilter);
    aFilterList.save;
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Clear of stored Filters didnt work');

    aFilterList.Read;
    Check(aFilterList.Count = 1, 'Wrong count of ' + IntToStr(aFilterList.Count) +' (should 1) after load of stored Filters');

    aTestFilter := aFilterList.Items[0];
    aClassname:=aTestFilter.ClassName;
    Check(aClassname='TFilterCopy', 'FilterCopy not correct loaded');

    aInStr.Clear;
    aInStr.Add('F1');
    aInStr.Add('F2');
    aOutStr.Clear;
    CheckTrue(aTestFilter.Execute(aInStr,aOutStr), 'Filtercopy Execute with correct filled in fields fails after save' );
    Check(aOutStr.Count=4, 'Execute Outfield have not the correct size (should 4) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty after save');
    Check(aOutStr[1]='', 'Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty after save');
    Check(aOutStr[2]='F1', 'Execute Outfield 2 false value (' + aOutStr[0] + ') after save');
    Check(aOutStr[3]='F2', 'Execute Outfield 3 false value (' + aOutStr[0] + ') after save');


  finally
    aInStr.Free;
    aOutStr.Free;
  end;
end;

procedure TTestCheckingBOM.TestFilterInputWithOkParams;
var
  aFilterList: TFilterList;
  aFilter, aTestFilter : TFilter;
  aClassname, aName : string;
  aInStr : TStringList;
  aOutStr : TStringList;
begin
  Log([ClassName],lsDebug);
  BOM_Filter.CreateTables;
  Check(GTIOPFManager.TableExists('Filters'),'Table Filters not in Database');
  aFilterList := gListManager.FilterList;
  aInStr := TStringList.Create;
  aOutStr := TStringList.Create;
  try
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Filters are not empty');

    aFilter := gFilterFactory.CreateInstance('FilterInput');
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterInput', 'FilterInput Classname not correct created');
    aName := aFilter.Filter;
    Check(aName='FilterInput', 'FilterInput Filter not correct created');
    CheckFalse(aFilter.CheckParams, 'FilterInput CheckParams with no params isnt false' );
    aFilter.InputFields:= '';
    aFilter.OutputFields:= '2';
    aFilter.Params:='ThisIsTheInput';
    CheckTrue(aFilter.CheckParams, 'FilterInput CheckParams with correct filled in fields fails' );
    aInStr.Clear;
    aInStr.Add('F1');
    aInStr.Add('F2');
    aInStr.Add('F3');
    aInStr.Add('F4');
    aInStr.Add('F5');
    aOutStr.Clear;
    CheckTrue(aFilter.Execute(aInStr,aOutStr), 'FilterInput Execute with correct filled in fields fails' );
    Check(aOutStr.Count=3, 'Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='', 'Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[2]='ThisIsTheInput', 'Execute Outfield 2 false value (' + aOutStr[0] + ')');

    aFilterList.Add(aFilter);
    aFilterList.save;
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Clear of stored Filters didnt work');

    aFilterList.Read;
    Check(aFilterList.Count = 1, 'Wrong count of ' + IntToStr(aFilterList.Count) +' (should 1) after load of stored Filters');

    aTestFilter := aFilterList.Items[0];
    aClassname:=aTestFilter.ClassName;
    Check(aClassname='TFilterInput', 'FilterInput not correct loaded');
    aInStr.Clear;
    aInStr.Add('F1');
    aInStr.Add('F2');
    aInStr.Add('F3');
    aInStr.Add('F4');
    aInStr.Add('F5');
    aOutStr.Clear;
    CheckTrue(aTestFilter.Execute(aInStr,aOutStr), 'FilterInput Execute reloaded with correct filled in fields fails after save' );
    Check(aOutStr.Count=3, 'Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='', 'Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[2]='ThisIsTheInput', 'Execute Outfield 2 false value (' + aOutStr[0] + ')');
  finally
    aInStr.Free;
    aOutStr.Free;
  end;
end;

procedure TTestCheckingBOM.TestFilterHashWithOkParams;
var
  aFilterList: TFilterList;
  aFilter, aTestFilter : TFilter;
  aClassname, aName : string;
  aInStr : TStringList;
  aOutStr : TStringList;
begin
  Log([ClassName],lsDebug);
  BOM_Filter.CreateTables;
  Check(GTIOPFManager.TableExists('Filters'),'Table Filters not in Database');
  aFilterList := gListManager.FilterList;
  aInStr := TStringList.Create;
  aOutStr := TStringList.Create;
  try
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Filters are not empty');

    aFilter := gFilterFactory.CreateInstance('FilterHash');
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterHash', 'FilterHash Classname not correct created');
    aName := aFilter.Filter;
    Check(aName='FilterHash', 'FilterHash Filter not correct created');
    CheckFalse(aFilter.CheckParams, 'FilterHash CheckParams with no params isnt false' );
    aFilter.InputFields:= '';
    aFilter.OutputFields:= '1';
    aFilter.Params:='123456789012345678901234567890';
    CheckTrue(aFilter.CheckParams, 'FilterHash CheckParams with correct filled in fields fails' );
    aInStr.Clear;
    aOutStr.Clear;
    CheckTrue(aFilter.Execute(aInStr,aOutStr), 'FilterHash Execute with correct filled in fields fails' );
    Check(aOutStr.Count=2, 'Execute Outfield have not the correct size (should 2) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='123456789012345678901234567890', 'Execute Outfield 1 false value (' + aOutStr[0] + ')');

    aFilterList.Add(aFilter);
    aFilterList.save;
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Clear of stored Filters didnt work');

    aFilterList.Read;
    Check(aFilterList.Count = 1, 'Wrong count of ' + IntToStr(aFilterList.Count) +' (should 1) after load of stored Filters');

    aTestFilter := aFilterList.Items[0];
    aClassname:=aTestFilter.ClassName;
    Check(aClassname='TFilterHash', 'TFilterHash not correct loaded');
    aInStr.Clear;
    aOutStr.Clear;
    CheckTrue(aTestFilter.Execute(aInStr,aOutStr), 'FilterHash Execute reloaded with correct filled in fields fails after save' );
    Check(aOutStr.Count=2, 'Execute Outfield have not the correct size (should 2) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='123456789012345678901234567890', 'Execute Outfield 1 false value (' + aOutStr[0] + ')');
  finally
    aInStr.Free;
    aOutStr.Free;
  end;
end;

procedure TTestCheckingBOM.TestFilterConvertNumberWithOkParams;
var
  aFilterList: TFilterList;
  aFilter, aTestFilter : TFilter;
  aClassname, aName : string;
  aInStr : TStringList;
  aOutStr : TStringList;
begin
  Log([ClassName],lsDebug);
  BOM_Filter.CreateTables;
  Check(GTIOPFManager.TableExists('Filters'),'Table Filters not in Database');
  aFilterList := gListManager.FilterList;
  aInStr := TStringList.Create;
  aOutStr := TStringList.Create;
  try
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Filters are not empty');

    aFilter := gFilterFactory.CreateInstance('FilterConvertNumber');
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterConvertNumber', 'FilterConvertNumber Classname not correct created');
    aName := aFilter.Filter;
    Check(aName='FilterConvertNumber', 'FilterConvertNumber Filter not correct created');
    CheckFalse(aFilter.CheckParams, 'FilterConvertNumber CheckParams with no params isnt false' );
    aFilter.InputFields:= '0';
    aFilter.OutputFields:= '2';
    aFilter.Params:='".",",",",","."';
    CheckTrue(aFilter.CheckParams, 'FilterConvertNumber CheckParams with correct filled in fields fails' );
    aInStr.Clear;
    aInStr.Add('1234,5');
    aInStr.Add('433.4');
    aInStr.Add('123');
    aOutStr.Clear;
    CheckFalse(aFilter.Execute(aInStr,aOutStr), 'FilterConvertNumber Execute with false filled in fields fails' );
    aFilter.InputFields:= '1';
    aOutStr.Clear;
    CheckTrue(aFilter.Execute(aInStr,aOutStr), 'FilterConvertNumber Execute with correct filled in fields fails' );
    Check(aOutStr.Count=3, 'Try 1 Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Try 1 Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='', 'Try 1 Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[2]='433,4', 'Try 1 Execute Outfield 2 false value (' + aOutStr[0] + ')');
    aFilter.InputFields:= '2';
    aOutStr.Clear;
    CheckTrue(aFilter.Execute(aInStr,aOutStr), 'FilterConvertNumber Execute with correct filled in fields fails' );
    Check(aOutStr.Count=3, 'Try 2 Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Try 2 Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='', 'Try 2 Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[2]='123', 'Try 2 Execute Outfield 2 false value (' + aOutStr[0] + ')');

    aFilterList.Add(aFilter);
    aFilterList.save;
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Clear of stored Filters didnt work');

    aFilterList.Read;
    Check(aFilterList.Count = 1, 'Wrong count of ' + IntToStr(aFilterList.Count) +' (should 1) after load of stored Filters');

    aTestFilter := aFilterList.Items[0];
    aClassname:=aTestFilter.ClassName;
    Check(aClassname='TFilterConvertNumber', 'FilterConvertNumber not correct loaded');
    aInStr.Clear;
    aInStr.Add('1234.5');
    aInStr.Add('433.4');
    aInStr.Add('123');
    aTestFilter.InputFields:= '0';
    aOutStr.Clear;
    CheckTrue(aTestFilter.Execute(aInStr,aOutStr), 'FilterConvertNumber Execute reloaded with correct filled in fields fails after save' );
    Check(aOutStr.Count=3, 'Try 3 Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='', 'Try 3 Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[1]='', 'Try 3 Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    Check(aOutStr[2]='1234,5', 'Try 3 Execute Outfield 2 false value (' + aOutStr[0] + ')');
  finally
    aInStr.Free;
    aOutStr.Free;
  end;
end;


function MOCKShowEditSearchDetailA(aData:TSearch; Zeile: string):TModalResult;
begin
  Result := mrAbort;
end;

function MOCKShowEditSearchDetailB(aData:TSearch; Zeile: string):TModalResult;
begin
  Result := mrAbort;
  aData.SearchText := '1a2b3c';
  aData.ResultText := 'PAAST';
  aData.Category := 'CAT1';
  aData.SubCategory := 'SUBCAT1';
  aData.Remark := 'Rem';
  Result:=mrOK;
end;

procedure TTestCheckingBOM.TestFilterSearchWithOkParams;
var
  aFilterList: TFilterList;
  aFilter, aTestFilter : TFilter;
  aClassname, aName : string;
  aInStr : TStringList;
  aOutStr : TStringList;
  // for a dummy search
  aSearchList : TSearchList;
  aSearch : TSearch;
begin
  Log([ClassName],lsDebug);
  // Create needed tables
  BOM_Filter.CreateTables;
  Check(GTIOPFManager.TableExists('Filters'),'Table Filters not in Database');
  BOM_Search.CreateTables;
  Check(GTIOPFManager.TableExists('Search'),'Table Search not in Database');
  // Create the object
  aFilterList := gListManager.FilterList;
  aInStr := TStringList.Create;
  aOutStr := TStringList.Create;
  try
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Filters are not empty');

    aFilter := gFilterFactory.CreateInstance('FilterSearch');
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterSearch', 'FilterSearch Classname not correct created');
    aName := aFilter.Filter;
    Check(aName='FilterSearch', 'FilterSearch Filter not correct created');
    (aFilter as TFilterSearch).OnCreateNewSearchElement := @MOCKShowEditSearchDetailA;
    CheckFalse(aFilter.CheckParams, 'FilterSearch CheckParams with no params isnt false' );
    aFilter.InputFields:= '0';
    aFilter.OutputFields:= '0,1,2';
    aFilter.Params:='none';
    CheckTrue(aFilter.CheckParams, 'FilterSearch CheckParams with correct filled in fields fails' );
    // Create a dummy Search
    aSearchList := TSearchList.CreateNew();
    aSearch := gSearchFactory.CreateInstance('Search');
    aSearch.SearchText := '1a2b3c';
    aSearch.ResultText := 'PAAST';
    aSearch.Category := 'CAT1';
    aSearch.SubCategory := 'SUBCAT1';
    aSearch.Remark := 'Rem';
    aSearch.dirty;
    aSearchList.Add(aSearch);
    //aSearchList.Save;
    //
    (aFilter as TFilterSearch).SearchList := aSearchList;
    aInStr.Clear;
    aInStr.Add('1234,5');
    aInStr.Add('433.4');
    aInStr.Add('123');
    aOutStr.Clear;
    CheckFalse(aFilter.Execute(aInStr,aOutStr), 'FilterSearch Execute with false filled in fields fails' );
    aInStr.Clear;
    aInStr.Add('1a2b3c');
    aInStr.Add('433.4');
    aInStr.Add('123');
    aOutStr.Clear;
    (aFilter as TFilterSearch).OnCreateNewSearchElement := @MOCKShowEditSearchDetailB;
    CheckTrue(aFilter.Execute(aInStr,aOutStr), 'FilterConvertNumber Execute with correct filled in fields fails' );
    Check(aOutStr.Count=3, 'Try 1 Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    Check(aOutStr[0]='PAAST', 'Try 1 Execute Outfield 0 false value (' + aOutStr[0] + ')');
    Check(aOutStr[1]='CAT1', 'Try 1 Execute Outfield 1 false value (' + aOutStr[0] + ')' );
    Check(aOutStr[2]='SUBCAT1', 'Try 1 Execute Outfield 2 false value (' + aOutStr[0] + ')');
    //aFilter.InputFields:= '2';
    //aOutStr.Clear;
    //CheckTrue(aFilter.Execute(aInStr,aOutStr), 'FilterConvertNumber Execute with correct filled in fields fails' );
    //Check(aOutStr.Count=3, 'Try 2 Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    //Check(aOutStr[0]='', 'Try 2 Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    //Check(aOutStr[1]='', 'Try 2 Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    //Check(aOutStr[2]='123', 'Try 2 Execute Outfield 2 false value (' + aOutStr[0] + ')');
    //
    aFilterList.Add(aFilter);
    aFilterList.save;
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Clear of stored Filters didnt work');

    aFilterList.Read;
    Check(aFilterList.Count = 1, 'Wrong count of ' + IntToStr(aFilterList.Count) +' (should 1) after load of stored Filters');
    //
    //aTestFilter := aFilterList.Items[0];
    //aClassname:=aTestFilter.ClassName;
    //Check(aClassname='TFilterConvertNumber', 'FilterConvertNumber not correct loaded');
    //aInStr.Clear;
    //aInStr.Add('1234.5');
    //aInStr.Add('433.4');
    //aInStr.Add('123');
    //aTestFilter.InputFields:= '0';
    //aOutStr.Clear;
    //CheckTrue(aTestFilter.Execute(aInStr,aOutStr), 'FilterConvertNumber Execute reloaded with correct filled in fields fails after save' );
    //Check(aOutStr.Count=3, 'Try 3 Execute Outfield have not the correct size (should 3) is ' +IntToStr(aOutStr.Count));
    //Check(aOutStr[0]='', 'Try 3 Execute Outfield 0 false value (' + aOutStr[0] + '), should be empty');
    //Check(aOutStr[1]='', 'Try 3 Execute Outfield 1 false value (' + aOutStr[0] + '), should be empty');
    //Check(aOutStr[2]='1234,5', 'Try 3 Execute Outfield 2 false value (' + aOutStr[0] + ')');
  finally
    aInStr.Free;
    aOutStr.Free;
    aSearchList.Free;
  end;
end;


procedure TTestCheckingBOM.TestFilterCRUD;
var
  aFilterList: TFilterList;
  aFilter : TFilter;
  aClassname : string;
begin
  Log([ClassName],lsDebug);
  BOM_Filter.CreateTables;
  Check(GTIOPFManager.TableExists('Filters'),'Table Filters not in Database');
  aFilterList := gListManager.FilterList;
  try
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Filters are not empty');

    aFilter := TFilter(gFilterFactory.CreateInstance('FilterCopy'));
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterCopy', 'Filtercopy not correct created');
    aFilterList.Add(aFilter);

    aFilter := TFilter(gFilterFactory.CreateInstance('FilterInput'));
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterInput', 'FilterInput not correct created');
    aFilterList.Add(aFilter);

    aFilter := TFilter(gFilterFactory.CreateInstance('FilterHash'));
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterHash', 'FilterHash not correct created');
    aFilterList.Add(aFilter);

    aFilter := TFilter(gFilterFactory.CreateInstance('FilterConvertNumber'));
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterConvertNumber', 'FilterConvertNumber not correct created');
    aFilterList.Add(aFilter);

    aFilter := TFilter(gFilterFactory.CreateInstance('FilterSearch'));
    aClassname:=aFilter.ClassName;
    Check(aClassname='TFilterSearch', 'FilterSearch not correct created');
    aFilterList.Add(aFilter);

    Check(aFilterList[0].ClassName='TFilterCopy', 'Filtercopy not correct Stored');
    Check(aFilterList[1].ClassName='TFilterInput', 'FilterInput not correct Stored');
    Check(aFilterList[2].ClassName='TFilterHash', 'FilterHash not correct Stored');
    Check(aFilterList[3].ClassName='TFilterConvertNumber', 'FilterConvertNumber not correct Stored');
    Check(aFilterList[4].ClassName='TFilterSearch', 'FilterSearch not correct Stored');

    Check(aFilterList.Count = 5, 'Wrong count of stored Filters');
    aFilterList.Save;
    aFilterList.Clear;
    Check(aFilterList.Count = 0, 'Clear of stored Filters didnt work');

    aFilterList.Read;
    Check(aFilterList.Count = 5, 'Wrong count of ' + IntToStr(aFilterList.Count) +' (should 5) after load of stored Filters');

  finally
    //
  end;

end;

procedure TTestCheckingBOM.SetUp;
begin
  BOM_Filter.DeleteTables;
  BOM_Hash.DeleteTables;
  BOM_Search.DeleteTables;
end;

procedure TTestCheckingBOM.TearDown;
begin
  BOM_Filter.DeleteTables;
  BOM_Hash.DeleteTables;
  BOM_Search.DeleteTables;
end;


procedure TTestCheckingBOM.TestLookForTableFiltersNotInDB;
begin
  Log([ClassName],lsDebug);
  CheckFalse(GTIOPFManager.TableExists('Filters'),'Table Filters should not be in Database');
end;




initialization

//  RegisterTest('WithDatabase.TestCheckingBOM',TTestCheckingBOM);
end.

