unit TestCheckingBOMHash_TST;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry
  , BOM_Filter
  , BOM_Hash
  , BOM_Search
  , uFilter
  , uListManager
  , tiOPFManager
  , tiLog
  ;

type

  { TTestCheckingBOMHash }

  TTestCheckingBOMHash= class(TTestCase)
  protected
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestLookForTableHashNotInDB;
    procedure TestCreateTableHashInBOM;
    procedure TestHashCRUD;

    procedure TestHashEdit;
    procedure TestHashDelete;

  end;

implementation


procedure TTestCheckingBOMHash.SetUp;
begin
  BOM_Filter.DeleteTables;
  BOM_Hash.DeleteTables;
  BOM_Search.DeleteTables;
end;

procedure TTestCheckingBOMHash.TearDown;
begin
  BOM_Filter.DeleteTables;
  BOM_Hash.DeleteTables;
  BOM_Search.DeleteTables;
end;

procedure TTestCheckingBOMHash.TestLookForTableHashNotInDB;
begin
  Log([ClassName],lsDebug);
  CheckFalse(GTIOPFManager.TableExists('Hashes'),'Table Hashes should not be in Database');
end;

procedure TTestCheckingBOMHash.TestCreateTableHashInBOM;
begin
  Log([ClassName],lsDebug);
  BOM_Hash.CreateTables;
  Check(GTIOPFManager.TableExists('Hashes'),'Table Hashes not in Database');
end;

procedure TTestCheckingBOMHash.TestHashCRUD;
var
  aHashList : THashList;
  aHash : THash;
  aClassname : string;
begin
  Log([ClassName],lsDebug);
  BOM_Hash.CreateTables;
  Check(GTIOPFManager.TableExists('Hashes'),'Table Hashes not in Database');
  aHashList := gListManager.HashList;
  try
    aHashList.Clear;
    Check(aHashList.Count = 0, 'HashList is not empty');

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy3';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    Check(aHashList.Count = 4, 'Wrong count of stored Hashes');
    aHashList.Save;
    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work');

    aHashList.Read;
    Check(aHashList.Count = 4, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 4) after load of stored Hashes');

    aHash := aHashList.Items[3];
    check(aHash.Filename = 'C:\Dummy3', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy1<');
  finally
    //
  end;
end;

procedure TTestCheckingBOMHash.TestHashEdit;
var
  aHashList : THashList;
  aHash : THash;
  aClassname : string;
begin
  Log([ClassName],lsDebug);
  BOM_Hash.CreateTables;
  Check(GTIOPFManager.TableExists('Hashes'),'Table Hashes not in Database');
  aHashList := gListManager.HashList;
  try
    aHashList.Clear;
    Check(aHashList.Count = 0, 'HashList is not empty');

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy0';
    aHash.Hash:='1234567890';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy1';
    aHash.Hash:='1234567891';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy2';
    aHash.Hash:='1234567892';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy3';
    aHash.Hash:='1234567893';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    Check(aHashList.Count = 4, 'Wrong count of stored Hashes');
    aHashList.Save;
    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work');

    aHashList.Read;
    Check(aHashList.Count = 4, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 4) after load of stored Hashes');

    aHash := aHashList.Items[0];
    check(aHash.Filename = 'C:\Dummy0', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy0<');
    check(aHash.Hash = '1234567890', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567890<');

    aHash := aHashList.Items[1];
    check(aHash.Filename = 'C:\Dummy1', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy1<');
    check(aHash.Hash = '1234567891', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567891<');

    aHash := aHashList.Items[2];
    check(aHash.Filename = 'C:\Dummy2', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy2<');
    check(aHash.Hash = '1234567892', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567892<');

    aHash := aHashList.Items[3];
    check(aHash.Filename = 'C:\Dummy3', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy3<');
    check(aHash.Hash = '1234567893', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567893<');

    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work');

    aHashList.Read;
    Check(aHashList.Count = 4, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 4) after load of stored Hashes');

    aHash := aHashList.Items[3];
    aHash.Filename:='C:\Dummy33';
    aHash.Hash:='12345678933';
    aHash.Dirty:=true;
    aHashList.Save;
    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work');

    aHashList.Read;
    Check(aHashList.Count = 4, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 4) after load of stored Hashes');

    aHash := aHashList.Items[0];
    check(aHash.Filename = 'C:\Dummy0', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy0<');
    check(aHash.Hash = '1234567890', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >1234567890<');

    aHash := aHashList.Items[1];
    check(aHash.Filename = 'C:\Dummy1', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy1<');
    check(aHash.Hash = '1234567891', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >1234567891<');

    aHash := aHashList.Items[2];
    check(aHash.Filename = 'C:\Dummy2', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy2<');
    check(aHash.Hash = '1234567892', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >1234567892<');

    aHash := aHashList.Items[3];
    check(aHash.Filename = 'C:\Dummy33', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy33<');
    check(aHash.Hash = '12345678933', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >12345678933<');

  finally
    //
  end;
end;


procedure TTestCheckingBOMHash.TestHashDelete;
var
  aHashList : THashList;
  aHash : THash;
  aClassname : string;
begin
  Log([ClassName],lsDebug);
  BOM_Hash.CreateTables;
  Check(GTIOPFManager.TableExists('Hashes'),'Table Hashes not in Database');
  aHashList := gListManager.HashList;
  try
    aHashList.Clear;
    Check(aHashList.Count = 0, 'HashList is not empty');

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy0';
    aHash.Hash:='1234567890';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy1';
    aHash.Hash:='1234567891';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy2';
    aHash.Hash:='1234567892';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    aHash := THash(gHashFactory.CreateInstance('Hash'));
    aHash.Filename:='C:\Dummy3';
    aHash.Hash:='1234567893';
    aHash.Dirty:=true;
    aClassname:=aHash.ClassName;
    Check(aClassname='THash', 'Hash not correct created');
    aHashList.Add(aHash);

    Check(aHashList.Count = 4, 'Wrong count of stored Hashes');
    aHashList.Save;
    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work');

    aHashList.Read;
    Check(aHashList.Count = 4, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 4) after load of stored Hashes');

    aHash := aHashList.Items[0];
    check(aHash.Filename = 'C:\Dummy0', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy0<');
    check(aHash.Hash = '1234567890', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567890<');

    aHash := aHashList.Items[1];
    check(aHash.Filename = 'C:\Dummy1', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy1<');
    check(aHash.Hash = '1234567891', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567891<');

    aHash := aHashList.Items[2];
    check(aHash.Filename = 'C:\Dummy2', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy2<');
    check(aHash.Hash = '1234567892', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567892<');

    aHash := aHashList.Items[3];
    check(aHash.Filename = 'C:\Dummy3', 'Wrong data stored -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy3<');
    check(aHash.Hash = '1234567893', 'Wrong data stored -> Field Hash is >' + aHash.Hash + '< expected >1234567893<');

    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work');

    aHashList.Read;
    Check(aHashList.Count = 4, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 4) after load of stored Hashes');

    aHash := aHashList.Items[3];
    aHash.Filename:='C:\Dummy33';
    aHash.Hash:='12345678933';
    aHash.Dirty:=true;
    aHashList.Save;
    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work');

    aHashList.Read;
    Check(aHashList.Count = 4, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 4) after load of stored Hashes');

    aHash := aHashList.Items[0];
    check(aHash.Filename = 'C:\Dummy0', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy0<');
    check(aHash.Hash = '1234567890', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >1234567890<');

    aHash := aHashList.Items[1];
    check(aHash.Filename = 'C:\Dummy1', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy1<');
    check(aHash.Hash = '1234567891', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >1234567891<');

    aHash := aHashList.Items[2];
    check(aHash.Filename = 'C:\Dummy2', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy2<');
    check(aHash.Hash = '1234567892', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >1234567892<');

    aHash := aHashList.Items[3];
    check(aHash.Filename = 'C:\Dummy33', 'Wrong data edited -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy33<');
    check(aHash.Hash = '12345678933', 'Wrong data edited -> Field Hash is >' + aHash.Hash + '< expected >12345678933<');

    //
    aHash := aHashList.Items[2];
    aHash.Deleted:=true;
    aHashList.FreeDeleted;
    Check(aHashList.Count = 3, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 3) after delete of a item');
    aHashList.Save;
    aHashList.Clear;
    Check(aHashList.Count = 0, 'Clear of stored Hashes didnt work (delete)');

    aHashList.Read;
    Check(aHashList.Count = 3, 'Wrong count of ' + IntToStr(aHashList.Count) +' (should 3) after delete and reread');

    aHash := aHashList.Items[2];
    check(aHash.Filename = 'C:\Dummy33', 'Wrong data after delted -> Field Filename is >' + aHash.Filename + '< expected >C:\Dummy33<');
    check(aHash.Hash = '12345678933', 'Wrong data after delted -> Field Hash is >' + aHash.Hash + '< expected >12345678933<');


  finally
    //
  end;
end;


initialization

  // RegisterTest(TTestCheckingBOMHash);
end.

