unit testcheckingbomsearch_tst;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry
  , BOM_Filter
  , BOM_Hash
  , BOM_Search
  , uFilter
  , uListManager
  , tiOPFManager
  , tiLog
  ;

type

  { TTestCheckingBOMSearch }

  TTestCheckingBOMSearch= class(TTestCase)
  protected
    procedure SetUp; override;
    procedure TearDown; override;
  public
    constructor CreateWithName(const AName: string); override;
  published
    procedure TestSearchLookForTableSearchNotInDB;
    procedure TestSearchCreateTableSearchInBOM;
    procedure TestSearchCRUD;
  end;

implementation

procedure TTestCheckingBOMSearch.SetUp;
begin
  BOM_Filter.DeleteTables;
  BOM_Hash.DeleteTables;
  BOM_Search.DeleteTables;
end;

procedure TTestCheckingBOMSearch.TearDown;
begin
  BOM_Filter.DeleteTables;
  BOM_Hash.DeleteTables;
  BOM_Search.DeleteTables;
end;

constructor TTestCheckingBOMSearch.CreateWithName(const AName: string);
begin
  Inherited CreateWithName('WithDatabase.'+AName);
end;

procedure TTestCheckingBOMSearch.TestSearchLookForTableSearchNotInDB;
begin
  Log([ClassName],lsDebug);
  CheckFalse(GTIOPFManager.TableExists('Search'),'Table Search should not be in Database');
end;

procedure TTestCheckingBOMSearch.TestSearchCreateTableSearchInBOM;
begin
  Log([ClassName],lsDebug);
  BOM_Search.CreateTables;
  Check(GTIOPFManager.TableExists('Search'),'Table Search not in Database');
end;

procedure TTestCheckingBOMSearch.TestSearchCRUD;
var
  aSearchList : TSearchList;
  aSearch : TSearch;
  aClassname : string;
begin
  Log([ClassName],lsDebug);
  BOM_Search.CreateTables;
  Check(GTIOPFManager.TableExists('Search'),'Table Search not in Database');
  aSearchList := gListManager.SearchList;
  try
    aSearchList.Clear;
    Check(aSearchList.Count = 0, 'SearchList is not empty');

    aSearch := TSearch(gSearchFactory.CreateInstance('Search'));
    aSearch.Dirty:=true;
    aClassname:=aSearch.ClassName;
    Check(aClassname='TSearch', 'Search not correct created');
    aSearchList.Add(aSearch);

    aSearch := TSearch(gSearchFactory.CreateInstance('Search'));
    aSearch.Dirty:=true;
    aClassname:=aSearch.ClassName;
    Check(aClassname='TSearch', 'Search not correct created');
    aSearchList.Add(aSearch);

    aSearch := TSearch(gSearchFactory.CreateInstance('Search'));
    aSearch.Dirty:=true;
    aClassname:=aSearch.ClassName;
    Check(aClassname='TSearch', 'Search not correct created');
    aSearchList.Add(aSearch);

    aSearch := TSearch(gSearchFactory.CreateInstance('Search'));
    aSearch.Dirty:=true;
    aClassname:=aSearch.ClassName;
    Check(aClassname='TSearch', 'Search not correct created');
    aSearchList.Add(aSearch);

    Check(aSearchList.Count = 4, 'Wrong count of stored Searches');
    aSearchList.Save;
    aSearchList.Clear;
    Check(aSearchList.Count = 0, 'Clear of stored Search didnt work');

    aSearchList.Read;
    Check(aSearchList.Count = 4, 'Wrong count of ' + IntToStr(aSearchList.Count) +' (should 4) after load of stored Searches');
  finally
    //
  end;
end;

initialization

  //RegisterTest(TTestCheckingBOMSearch);
end.

